/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/


package edu.lehigh.swat.hawk.core;

/**
 * A general representation of the Properties.
 * implementations depend on the storage models</p>
 * @author Z. Pan
 * @version 1.0
 */

public interface OntProperty extends OntObject{


  /**
   * get the domain
   */
  public ObjectId getDomain();

  /**
   * set the domain
   */
  public void setDomain(ObjectId c);

  /**
   * get the range
   */
  public ObjectId getRange();

  /**
   * set the domain
   */
  public void setRange(ObjectId c);

  /**
   * test if this property is transitive
   */
  public boolean isTransitive();

  /**
   * set the property to be transitive or not
   */
  public void setTransitive(boolean b);

  /**
   * test if this property is symetric
   */
  public boolean isSymmetric();

  /**
   * set the property to be symetric or not
   */
  public void setSymmetric(boolean b);

  /**
   * add a direct subProperty
   */
  public void addSubProperty(ObjectId p);

  /**
   * get the direct subProperties
   */
  public ObjectId[] getDirectSubProperties();

  /**
   * delete a direct subProperty
   */
  public void deleteSubProperty(ObjectId p);

  /**
   * add a direct super Property
   */
  public void addSuperProperty(ObjectId p);

  /**
   * get the direct super Properties
   */
  public ObjectId[] getDirectSuperProperties();

  /**
   * delete a direct super Property
   */
  public void deleteSuperProperty(ObjectId p);

  /**
   * Test if this property is deprecated.
   */
  public boolean isDeprecated();

  /**
   * Set the property to be deprecated.
   */
  public void setDeprecated(boolean dep);

  /**
   * Test if this property is functional.
   */
  public boolean isFunctional();

  /**
   * Set the property to be functional.
   */
  public void setFunctional(boolean dep);

  /**
   * Test if this property is inverse functional.
   */
  public boolean isInverseFunctional();

  /**
   * Set the property to be inverse functional.
   */
  public void setInverseFuncional(boolean dep);

  /**
   * Add a 'inverseOf' property
   */
  public void addInverseOf(ObjectId p);

  /**
   * delete a 'inverseOf' property
   */
  public void deleteInverseOf(ObjectId p);

  /**
   * get the 'inverseOf' properties
   */
  public ObjectId[] getInverseOf();

  /**
   * Add a equivalent property.
   */
  public void addEquivalentProperty(ObjectId ec);

  /**
   * Delete a equivalent property.
   */
  public void deleteEquivalentProperty(ObjectId r);

  /**
   * Get the equivalent properties.
   */
  public ObjectId[] getEquivalentProperties();

  /**
   * Tell this property is ObjectProperty or DatatypeProperty.
   * Null means unknown.
   */
  public Boolean isDatatype();

  /**
   * Set this property to be DatatypeProperty or ObjectProperty.
   */
  public void setDatatype(boolean b);

  /**
   * Tell this property is AnnotationProperty or not.
   *
   */
  public boolean isAnnotation();

  /**
   * Set this property to be DatatypeProperty or not.
   */
  public void setAnnotation(boolean b);

}