/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/


package edu.lehigh.swat.hawk.core;
import java.util.Iterator;
/**
 * encapsulating the web ontology </p>
 * @author Z. Pan
 * @version 1.0
 */

public interface Ontology extends Document{


  /**
   * add a class to this ontology
   */
  public void addOntClass(OntClass c);

  /**
   * get the specified class
   */
  public OntClass getOntClass(ObjectId id);

  /**
   * delete the specified class
   */
  public void deleteOntClass(ObjectId id);

  /**
   * get all the classes in this ontology
   */
  public OntClass[] getAllClasses();

  /**
   * Get the Iterator of all the classes.
   */
  public Iterator getClassIterator();

  /**
   * add a property to this ontology
   */
  public void addProperty(OntProperty p);

  /**
   * get the specified property
   */
  public OntProperty getProperty(ObjectId id);

  /**
   * delete a specified property
   */
  public void deleteProperty(ObjectId id);

  /**
   * get all the properties in this ontology
   */
  public OntProperty[] getAllProperties();

  /**
   * Get the Iterator of all the properties.
   */
  public Iterator getPropertyIterator();

  /**
   * add a property to this ontology
   */
  public void addRestriction(PropertyRestriction p);

  /**
   * get the specified property
   */
  public PropertyRestriction getRestriction(ObjectId id);

  /**
   * delete a specified property
   */
  public void deleteRestriction(ObjectId id);

  /**
   * get all the properties in this ontology
   */
  public PropertyRestriction[] getAllRestrictions();

  /**
   * Get the priorVersion ontologies which are backwardCompatibleWith
   */
  public DocId[] getCompatiblePriorOnts();

  /**
   * Add a priorVersion ontology which is backwardCompatibleWith.
   */
  public void addCompatiblePriorOnts(DocId id);

  /**
   * Delete a priorVersion ontology which is backwardCompatibleWith.
   */
  public void deleteCompatiblePriorOnts(DocId id);

  /**
   * Get the priorVersion ontologies may be incompatibleWith
   */
  public DocId[] getPriorOnts();

  /**
   * Add a priorVersion ontology may be incompatibleWith.
   */
  public void addPriorOnts(DocId id);

  /**
   * Delete a priorVersion ontology may be incompatibleWith.
   */
  public void deletePriorOnts(DocId id);

  /**
   * get the version of this ontology
   */
  public String getVersion();

  /**
   * Set the version of this ontology.
   */
  public void setVersion(String ver);

  /**
   * Create a Class object.
   */
  public OntClass createClass();

  /**
   * Create a Property object.
   */
  public OntProperty createProperty();

  /**
   * Create a PropertyRestriction object.
   */
  public PropertyRestriction createRestriction();

  /**
   * To see if ontology has the class.
   */
  public boolean hasClass(String name);

  /**
   * To see if ontology has the property.
   */
  public boolean hasProperty(String name);

  /**
   * Get All the classes plus the imported Classes.
   */
  public OntClass[] getAllPlusImportedClasses();

  /**
   * Get All the classes plus the imported properties.
   */
  public OntProperty[] getAllPlusImportedProperties();

  /**
   * To tell if the ontology imorted specified class.
   */
  public boolean importedClass(ObjectId cid);

  /**
   * To tell if the ontology imported specified property.
   */
  public boolean importedProperty(ObjectId pid);
}