/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/


package edu.lehigh.swat.hawk.owl;
import java.io.*;
import edu.lehigh.swat.hawk.core.*;
import edu.lehigh.swat.hawk.storage.StorageModel;
import com.hp.hpl.jena.rdf.arp.JenaReader;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.vocabulary.RDFS;
import com.hp.hpl.jena.vocabulary.OWL;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.rdf.model.impl.SelectorImpl;
import java.util.*;


/**
 * OWL reader, reads OWL documents.
 * @author Z. Pan
 * @version 1.0
 */

public class OwlRdfReader extends OntReader{

  /**
   * Constructor
   */
  public OwlRdfReader(StorageModel s) throws Exception{
    super();
    smodel = s;
    try{
      resource = Class.forName("com.hp.hpl.jena.rdf.model.Resource");
    }catch(Exception e){
      e.printStackTrace();
    }
  }

  /**
   * Use empty string as XMl base.
   */
  public Document read(InputStream in)throws Exception{
    return this.read(in,"");
  }

  /**
   * Read document from Reader.
   */
/*  public Document read(Reader reader, String xmlBase){
    Document d = smodel.getDocument(xmlBase);
    if(d!=null)
      return d;

    PreParser pp = new PreParser(reader);
    HashMap map = pp.getNamespaces();
    if(xmlBase=="")
      xmlBase=pp.getXMLBase();
// HashMap map = new HashMap();
    JenaReader jenaReader = new JenaReader();
    Model model = ModelFactory.createDefaultModel();
    jenaReader.read(model, reader, xmlBase);
    return readJenaModel(model, xmlBase, map);
  }
*/

  /**
   * Read document from given URL
   */
  public Document read(String url) throws Exception{
    if(url.equalsIgnoreCase("http://www.w3.org/2002/07/owl")||
        url.equalsIgnoreCase("http://www.w3.org/2000/01/rdf-schema")||
         url.equalsIgnoreCase("http://www.w3.org/1999/02/22-rdf-syntax-ns"))
      return null;

    PreParser pp = new PreParser(url);
    HashMap map = pp.getNamespaces();
    String xmlBase = pp.getXMLBase();
    if(xmlBase.equals(""))
      xmlBase = url;
    Document d = smodel.getDocument(xmlBase);
    if(d!=null)
      return d;
    JenaReader jenaReader = new JenaReader();
    Model model = ModelFactory.createDefaultModel();
    jenaReader.read(model,url);
    return readJenaModel(model, xmlBase, map);
  }


  /**
   * Read an input in OWL format to the model and return the document object
   */
  public Document read(InputStream in, String xmlBase) throws Exception{
    //long t0= System.currentTimeMillis(); //temp
    ByteArrayInputStream bin=null, bin2=null;
    ByteArrayOutputStream bout = new ByteArrayOutputStream();
 //   try{
      byte[] buf = new byte[10240];
      int len = 0;
      while((len = in.read(buf))>0){
        bout.write(buf,0,len);
      }
      //System.out.println("inputstream begin: \n"+bout.toString()+"\n!!!the end!!!");
      //len = bout.size();
      //buf = new byte[len];
      buf = bout.toByteArray();
      bin = new ByteArrayInputStream(buf);
      bin2 = new ByteArrayInputStream(buf);

//    }catch (Exception e) {
//        e.printStackTrace();
//    }
//try{
    PreParser pp = new PreParser(bin);
    HashMap map = pp.getNamespaces();
    if(xmlBase=="")
      xmlBase=pp.getXMLBase();
  //}catch(Exception e){};
    Document d = smodel.getDocument(xmlBase);
    if(d!=null)
      return d;
//   HashMap map = new HashMap();
    JenaReader jenaReader = new JenaReader();
   //  long t1= System.currentTimeMillis(); //temp
    Model model = ModelFactory.createDefaultModel();
        //long t2 = System.currentTimeMillis(); //temp
    jenaReader.read(model, bin2, xmlBase);
    //long t3 = System.currentTimeMillis() - t2; //temp
    //System.out.println("jena time: "+t3);
//System.out.println(t1-t0);
  //  System.out.println(t2-t1);
 // System.out.println(xmlBase);
    return readJenaModel(model, xmlBase,map);
  }

  /**
   * Process the document from the triple-based model in Jena.
   * @param jenaReader
   * @param model
   * @param xmlBase
   * @return
   */
  private Document readJenaModel(Model model,
                                 String xmlBase, HashMap nsMap) throws Exception{
    Resource itself = model.getResource(xmlBase);
    Document doc = null;
    Ontology ont = null;

    ResIterator ti = model.listSubjectsWithProperty(RDF.type,OWL.Ontology);
    while(ti.hasNext())
    {
      Resource r = ti.nextResource();
      if(r!=null&&r.getURI()!=null&&!r.getURI().equals(""))
          xmlBase = r.getURI().replace('#',' ').trim();
    } // z. pan on 6/29/05: some ontologies don't have base but give info in header.



/*    if(itself!=null&&model.contains(itself, RDF.type, OWL.Ontology)){
      ont = smodel.createOntology();
      doc = ont;
    }
    else{
      doc = smodel.createDocument();
    }
*/  //better way to identify ontology/document
    boolean isOnto = false;
    ResIterator rirc = model.listSubjectsWithProperty(RDF.type, RDFS.Class);
    ResIterator ric = model.listSubjectsWithProperty(RDF.type, OWL.Class);
    ResIterator riop = model.listSubjectsWithProperty(RDF.type,OWL.ObjectProperty);
    ResIterator ridp = model.listSubjectsWithProperty(RDF.type, OWL.DatatypeProperty);
    ResIterator rip = model.listSubjectsWithProperty(RDF.type, RDF.Property);
    ResIterator rion = model.listSubjectsWithProperty(RDF.type, OWL.Ontology);
    if(rirc==null||ric==null||riop==null||ridp==null||rip==null||rion==null)
      isOnto = false;
    else if(ric.hasNext()||riop.hasNext()||ridp.hasNext()||(rip.hasNext()&&rion.hasNext())
        ||(rirc.hasNext()&&rion.hasNext()))
      isOnto = true;

    if(isOnto){
      ont = smodel.createOntology();
      doc = ont;
    }
    else{
      doc = smodel.createDocument();
    }

    DocId docid = new DocId(xmlBase);
    doc.setID(docid);

    smodel.addDocument(doc);
//    HashMap ns = jenaReader.getNameSpaces();
    doc.setNameSpaces(nsMap);
    doc.setXMLBase(xmlBase);
//  System.out.println(xmlBase);
    NodeIterator ni = model.listObjectsOfProperty(itself,RDFS.comment);
    while(ni.hasNext()){
        Literal comment = (Literal)ni.nextNode();
        doc.addDescription(comment.getLexicalForm());
    }

    ni = model.listObjectsOfProperty(itself,RDFS.label);
    if(ni.hasNext()){
        Literal label = (Literal)ni.nextNode();
        doc.setLabel(label.getLexicalForm());
    }

    ni = model.listObjectsOfProperty(OWL.imports);
    while(ni.hasNext()){
      Resource l = (Resource)ni.nextNode();
      DocId dId = new DocId(l.getURI());
      if(this.getLoadImports())
        this.read(dId.getId());
      doc.addImports(dId);
    }

    if(ont!=null){
      ni = model.listObjectsOfProperty(itself,OWL.versionInfo);
      if(ni.hasNext()){
        Literal ver = (Literal)ni.nextNode();
        ont.setVersion(ver.getLexicalForm());
      }

      ni = model.listObjectsOfProperty(OWL.priorVersion);
      while(ni.hasNext()){
        Resource l = (Resource)ni.nextNode();
        DocId dId = new DocId(l.getURI());
        ont.addPriorOnts(dId);
      }

      ni = model.listObjectsOfProperty(OWL.backwardCompatibleWith);
      while(ni.hasNext()){
        Resource l = (Resource)ni.nextNode();
        DocId dId = new DocId(l.getURI());
        this.read(l.getURI());
        ont.addCompatiblePriorOnts(dId);
      }

      //OWL.incompatibleWith??

      readOntology(ont, model);
      ont.flush();
      return ont;

   }// end if isOntology

   else{
     readIndividuals(doc,model);
     doc.flush();
     return doc;
   }
  }

  /**
   * Parse and construct an ontology.
   */
  protected void readOntology(Ontology ont, Model model){
    long t0 = System.currentTimeMillis();//test
    readClasses(ont, model);
      long t1 = System.currentTimeMillis()-t0; //test
//      System.out.println("class: "+t1); //test

      t0 = System.currentTimeMillis();//test
    readProperties(ont, model);
       t1 = System.currentTimeMillis()-t0; //test
//      System.out.println("property: "+t1); //test

      t0 = System.currentTimeMillis();//test
    readRestrictions(ont,model);
       t1 = System.currentTimeMillis()-t0; //test
 //   System.out.println("restriction: "+t1); //test

      t0 = System.currentTimeMillis();//test
    readAxioms(ont, model);
     t1 = System.currentTimeMillis()-t0; //test
 //   System.out.println("axiom: "+t1); //test
      t0 = System.currentTimeMillis();//test
      readIndividuals(ont, model);
      t1 = System.currentTimeMillis()-t0; //test
//     System.out.println("individual: "+t1); //test

//    descendants.pop();
  }

  /**
   * Construct classes and add to the ontology.
   */
  private void readClasses(Ontology ont, Model model){

    ResIterator ri = model.listSubjectsWithProperty(RDF.type,OWL.Class);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      addClass(ont,r,"",getComment(r,model),getLabel(r,model));
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.DeprecatedClass);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      addClass(ont,r,"deprecated",getComment(r,model),getLabel(r,model));
    }

    ri = model.listSubjectsWithProperty(RDF.type,RDFS.Class);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      addClass(ont,r,"",getComment(r,model),getLabel(r,model));
    }

  }

  /**
   * Construct properties and add to the ontology.
   */
  private void readProperties(Ontology ont, Model model){   //properties
//    long t0 = System.currentTimeMillis();//test

    ResIterator ri = model.listSubjectsWithProperty(RDF.type,OWL.DatatypeProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,null);
    }
    ri = model.listSubjectsWithProperty(RDF.type,OWL.ObjectProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,new Boolean(false));
    }

    ri = model.listSubjectsWithProperty(RDF.type,RDF.Property);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,new Boolean(true));
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.AnnotationProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,null);
      if(op!= null){
        op.setAnnotation(true);
      }
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.DeprecatedProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,null);
      if(op != null){
        op.setDeprecated(true);
      }
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.TransitiveProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,new Boolean(false));
      if(op != null){
        op.setTransitive(true);
      }
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.SymmetricProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,new Boolean(false));
      if(op != null){
        op.setSymmetric(true);
      }
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.FunctionalProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,new Boolean(false));
      if(op != null){
        op.setFunctional(true);
      }
    }

    ri = model.listSubjectsWithProperty(RDF.type,OWL.InverseFunctionalProperty);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      OntProperty op = addProperty(ont,r,model,new Boolean(false));
      if(op != null){
        op.setInverseFuncional(true);
      }
    }

  }

  /**
   * Construct individuals and add to the ontology.
   */
  private void readIndividuals(Document doc, Model model){ //individuals
    StmtIterator si = model.listStatements(new SelectorImpl(null,RDF.type,(RDFNode)null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource obj = (Resource)stmt.getObject();
      ObjectId oid = resourceToId(obj);
      if(oid==null)
        continue;
      if(!isVocabularyResource(obj)||obj.equals(OWL.Thing)){
        Resource sub = (Resource)stmt.getSubject();
        ObjectId iid = resourceToId(sub);
        Individual indiv = doc.getIndividual(iid);
   //     System.out.println(iid.getId());//test
        if(indiv==null){
          indiv = doc.createIndividual();
          indiv.setID(iid);
          indiv.addDescription(getComment(sub,model));
          indiv.setLabel(getLabel(sub,model));
          indiv.addSourceClass(oid);
          doc.addIndividual(indiv);
        }else{
          indiv.addSourceClass(oid);
          continue;
        }
        StmtIterator isi = model.listStatements(new SelectorImpl(sub,null,
            (RDFNode) null));
        while(isi.hasNext()){
          Statement st = isi.nextStatement();
          Property p = st.getPredicate();
          ObjectId pid = resourceToId(p);
          if(pid==null||p.equals(RDF.type)||p.equals(RDFS.comment)||p.equals(RDFS.label))
            continue;

          RDFNode vn = st.getObject();
          if(vn instanceof Resource){
            Resource vr = (Resource)vn;
            ObjectId vid = resourceToId(vr);
            indiv.addProperty(pid,vid);
          }
          else if(vn instanceof Literal)
          {
            Literal vl = (Literal)vn;
            String s = vl.getLexicalForm();
            String dtype;
            if(vl.getDatatype()!=null)
              dtype = vl.getDatatype().getURI();
            else
              dtype = "";
            DatatypeValue dv = new DatatypeValue(s,dtype);
            indiv.addProperty(pid,dv);
          }
        }
      }
    }

/*
    ResIterator ti = model.listSubjects();
    while(ti.hasNext()){
      Resource r = ti.nextResource();
      NodeIterator obji = model.listObjectsOfProperty(r,RDF.type);
      if(!obji.hasNext()){
        ObjectId oid = resourceToId(r);
      }
    }
*/

    //sameAs
    si = model.listStatements(new SelectorImpl(null,OWL.sameAs,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
/*      HashSet set = new HashSet();
      set.add(sid);
      set.add(oid);
      ont.addSameIndividualsSet(set);
*/    Individual i1 = doc.getIndividual(sid);
      Individual i2 = doc.getIndividual(oid);
      if(i1!=null&&i2!=null){
        i1.addSameAs(oid);
        i2.addSameAs(sid);
        Collection c1 = i1.getSameSet();
        Collection c2 = i2.getSameSet();
        if(!c1.equals(c2)){
          Collection c = combineSets(c1,c2);
          doc.deleteSameIndividualsSet(c1);
          doc.deleteSameIndividualsSet(c2);
          doc.addSameIndividualsSet(c);
          i1.setSameSet(c);
          i2.setSameSet(c);
        }
      }
    }
//differentFrom
    si = model.listStatements(new SelectorImpl(null,OWL.differentFrom,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      HashSet set = new HashSet();
      set.add(sid);
      set.add(oid);
      doc.addDistinctIndividualsSet(set);
      Individual i1 = doc.getIndividual(sid);
      Individual i2 = doc.getIndividual(oid);
      if(i1!=null&&i2!=null){
        i1.addDifferentFrom(oid);
        i2.addDifferentFrom(sid);
      }
    }

    //AllDifferent
    ResIterator ri = model.listSubjectsWithProperty(RDF.type,OWL.AllDifferent);
    while(ri.hasNext()){
      Resource ad = ri.nextResource();
      NodeIterator ni = model.listObjectsOfProperty(ad, OWL.distinctMembers);
      Resource members = (Resource) firstNode(ni);
      if(!model.contains(members,RDF.type,RDF.List))
        continue;
      Collection c = unpackList(model,members);
      Iterator it = c.iterator();
      if(!c.isEmpty()){
        Collection cc = new ArrayList();
        while(it.hasNext()){
          Resource re = (Resource)it.next();
          ObjectId oid = resourceToId(re);
          if(oid==null)
            continue;
          cc.add(oid);
          if(doc.getIndividual(oid)==null){
            Individual indiv = doc.createIndividual();
            indiv.setID(oid);
            NodeIterator ndi= model.listObjectsOfProperty(re,RDF.type);
            Resource rsc = (Resource)firstNode(ndi);
            ObjectId scid = resourceToId(rsc);
            indiv.addSourceClass(scid);
            doc.addIndividual(indiv);
          }
        }
        doc.addDistinctIndividualsSet(cc);
      }
    }

  }

  /**
   * Construct restrictions and add to the ontology.
   */
  private void readRestrictions(Ontology ont, Model model){//Restriction
    long t0 = System.currentTimeMillis();//test
    ResIterator ri = model.listSubjectsWithProperty(RDF.type,OWL.Restriction);
    while(ri.hasNext()){
      Resource r = ri.nextResource();
      ObjectId rid = resourceToId(r);
      if(rid==null)
        continue;
      PropertyRestriction res = ont.getRestriction(rid);
      if(res==null){
        res = ont.createRestriction();
        res.setID(rid);
        ont.addRestriction(res);
      }
      NodeIterator ni = model.listObjectsOfProperty(r,OWL.onProperty);
      if(!ni.hasNext())
        continue;
      Resource pr = (Resource)ni.nextNode();
      ObjectId opid = resourceToId(pr);
      res.setOnProperty(opid);

      ni = model.listObjectsOfProperty(r,OWL.allValuesFrom);
      if(ni.hasNext()){
        pr = (Resource)ni.nextNode();
        opid = resourceToId(pr);
        res.setAllValuesFrom(opid);
      }

      ni = model.listObjectsOfProperty(r,OWL.someValuesFrom);
      if(ni.hasNext()){
        pr = (Resource)ni.nextNode();
        opid = resourceToId(pr);
        res.setSomeValuesFrom(opid);
      }

      ni = model.listObjectsOfProperty(r,OWL.hasValue);
      if(ni.hasNext()){
        RDFNode pd = ni.nextNode();
        if(pd.canAs(resource)){
          pr = (Resource)pd;
          opid = resourceToId(pr);
          if(ont.getIndividual(opid)!=null)
          res.setHasValue(opid);
        }
        else{
          Literal pl = (Literal)pd;
          if(pl.isLiteral()){
            String s = pl.getLexicalForm();
            DatatypeValue dv = new DatatypeValue(s,"");
            res.setHasValue(dv);
          }
        }
      }

      ni = model.listObjectsOfProperty(r,OWL.cardinality);
      Literal cl;
      if(ni.hasNext()){
        cl = (Literal)ni.nextNode();
        String ls = cl.getLexicalForm();
        int card = Integer.parseInt(ls);
        res.setMinCardinality(card);
        res.setMaxCardinality(card);
      }

      ni = model.listObjectsOfProperty(r,OWL.maxCardinality);
      if(ni.hasNext()){
        cl = (Literal)ni.nextNode();
        String ls = cl.getLexicalForm();
        int card = Integer.parseInt(ls);
        res.setMaxCardinality(card);
      }

      ni = model.listObjectsOfProperty(r,OWL.minCardinality);
      if(ni.hasNext()){
        cl = (Literal)ni.nextNode();
        String ls = cl.getLexicalForm();
        int card = Integer.parseInt(ls);
        res.setMinCardinality(card);
      }
    }
  }

  /**
   * Construct axioms and add to the ontology.
   */
  private void readAxioms(Ontology ont, Model model){  //axioms
    long t0 = System.currentTimeMillis();//test
    StmtIterator si = model.listStatements(new SelectorImpl(null,RDFS.subClassOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      ObjectId sid = resourceToId(sub);

      Resource obj = (Resource)stmt.getObject();
      ObjectId oid = resourceToId(obj);

      if(sid==null || oid==null)
        continue;

      OntClass subc = ont.getOntClass(sid);
      OntClass superc = ont.getOntClass(oid);
      if(subc!=null)
        subc.addSuperClass(oid);
      if(superc!=null)
        superc.addSubClass(sid);
    }

    si = model.listStatements(new SelectorImpl(null,RDFS.subPropertyOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntProperty subc = ont.getProperty(sid);
      OntProperty superc = ont.getProperty(oid);
      if(subc!=null)
        subc.addSuperProperty(oid);
      if(superc!=null)
        superc.addSubProperty(sid);
    }

    si = model.listStatements(new SelectorImpl(null,RDFS.domain,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntProperty subc = ont.getProperty(sid);
      if(subc!=null)
        subc.setDomain(oid);
    }

    si = model.listStatements(new SelectorImpl(null,RDFS.range,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = null;
      RDFNode objnode  = stmt.getObject();
      if(objnode instanceof Resource){
        Resource obj = (Resource)stmt.getObject();
        oid = resourceToId(obj);
      }else if(objnode instanceof Literal){
            Literal vl = (Literal)objnode;
            String s = vl.getLexicalForm();
            String dtype;
            if(vl.getDatatype()!=null)
              dtype = vl.getDatatype().getURI();
            else
              dtype = "";
            DatatypeValue dv = new DatatypeValue(s,dtype);
            oid = dv;
          }

      if(sid==null || oid==null)
        continue;
      OntProperty subc = ont.getProperty(sid);
      if(subc!=null)
        subc.setRange(oid);
    }

    si = model.listStatements(new SelectorImpl(null,OWL.equivalentClass,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntClass subc = ont.getOntClass(sid);
      OntClass superc = ont.getOntClass(oid);
      if(subc!=null)
        subc.addEquivalentClass(oid);
      if(superc!=null)
        superc.addEquivalentClass(sid);
    }

    si = model.listStatements(new SelectorImpl(null,OWL.equivalentProperty,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntProperty subc = ont.getProperty(sid);
      OntProperty superc = ont.getProperty(oid);
      if(subc!=null)
        subc.addEquivalentProperty(oid);
      if(superc!=null)
        superc.addEquivalentProperty(sid);
    }

    si = model.listStatements(new SelectorImpl(null,OWL.inverseOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntProperty subc = ont.getProperty(sid);
      OntProperty superc = ont.getProperty(oid);
      if(subc!=null)
        subc.addInverseOf(oid);
      if(superc!=null)
        superc.addInverseOf(sid);
    }

    si = model.listStatements(new SelectorImpl(null,OWL.disjointWith,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntClass subc = ont.getOntClass(sid);
      OntClass superc = ont.getOntClass(oid);
      if(subc!=null)
        subc.addDisjointWith(oid);
      if(superc!=null)
        superc.addDisjointWith(sid);
    }

    si = model.listStatements(new SelectorImpl(null,OWL.complementOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      ObjectId oid = resourceToId(obj);
      if(sid==null || oid==null)
        continue;
      OntClass subc = ont.getOntClass(sid);
      OntClass superc = ont.getOntClass(oid);
      if(subc!=null)
        subc.addComplementOf(oid);
      //is restriction already be anon class in jena?
    }

    si = model.listStatements(new SelectorImpl(null,OWL.intersectionOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      if(sid==null)
        continue;
//      if(!model.contains(obj,RDF.type,RDF.Bag))
//        continue;
      Collection c = unpackList(model,obj);
      Iterator it = c.iterator();
      OntClass subc = ont.getOntClass(sid);
      if(subc!=null&&!c.isEmpty()){
          while(it.hasNext()){
            Resource re = (Resource)it.next();
            ObjectId oid = resourceToId(re);
            if(oid==null)
              continue;
            subc.addIntersectionOf(oid);
          }
        }
      }


    si = model.listStatements(new SelectorImpl(null,OWL.unionOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      if(sid==null)
        continue;
      if(!model.contains(obj,RDF.type,RDF.List))
        continue;
      Collection c = unpackList(model,obj);
      Iterator it = c.iterator();
      OntClass subc = ont.getOntClass(sid);
      if(subc!=null&&!c.isEmpty()){
          while(it.hasNext()){
            Resource re = (Resource)it.next();
            ObjectId oid = resourceToId(re);
            if(oid==null)
              continue;
            subc.addUnionOf(oid);
          }
        }
      }

    si = model.listStatements(new SelectorImpl(null,OWL.oneOf,
        (RDFNode) null));
    while(si.hasNext()){
      Statement stmt = si.nextStatement();
      Resource sub = (Resource)stmt.getSubject();
      Resource obj = (Resource)stmt.getObject();
      ObjectId sid = resourceToId(sub);
      if(sid==null)
        continue;
      if(!model.contains(obj,RDF.type,RDF.List))
        continue;
      Collection c = unpackList(model,obj);
      Iterator it = c.iterator();
      OntClass subc = ont.getOntClass(sid);
      if(subc!=null&&!c.isEmpty()){
          while(it.hasNext()){
            Resource re = (Resource)it.next();
            ObjectId oid = resourceToId(re);
            if(oid==null)
              continue;
            subc.addOneOf(oid);
          }
        }
      }

  }



    /**
     * OilEd: Unpacks a RDF List into a collection of resources
     **/
  protected Collection unpackList(Model currentModel, Resource r) {
	    Collection result = new ArrayList();
	    /* Grab the first thing in the list */
	    Resource current = r;
	    int firstCount = count(currentModel.listObjectsOfProperty(r, RDF.first));
	    NodeIterator firsts = currentModel.listObjectsOfProperty(r, RDF.first);
	    while (firsts!=null && firstCount==1) {
        Resource listHead = (Resource) firstNode(firsts);
        result.add(listHead);
		/* Find the next bit of the list */
        NodeIterator rests = currentModel.listObjectsOfProperty(current, RDF.rest);
        int restCount = count(currentModel.listObjectsOfProperty(current, RDF.rest));

        if (restCount==0) {
		    /* There's nothing else. Terminate the loop */
          firsts = null;
        }
        else {
          current = (Resource) firstNode(rests);

          if (currentModel.contains(current, RDF.type, RDF.nil)) {
			/* If the rest of the list is empty, terminate the loop */
            firsts = null;
          }
          else {
			/* otherwise, move along the list */
            firsts = currentModel.listObjectsOfProperty(current, RDF.first);
            firstCount = count(currentModel.listObjectsOfProperty(current, RDF.first));
          }
        }
	    }
	    return result;
  }

  /**
   * OilEd: Returns the first statement in a model. Assumes there is one, so
	 * you've better have checked that before calling this.
   */
  protected RDFNode firstNode(NodeIterator ri){
    if (ri.hasNext()) {
      return ri.nextNode();
    }
    else {
      return null;
    }
  }

  /**
   * Get the comment of a resource.
   */
  private String getComment(Resource r, Model model){
    NodeIterator ni = model.listObjectsOfProperty(r,RDFS.comment);
    if(ni.hasNext()){
      Literal l = (Literal)firstNode(ni);
      return l.getLexicalForm();
    }
    else
      return null;
  }

  /**
   * Get the label of a resource.
   */
  private String getLabel(Resource r, Model model){
    NodeIterator ni = model.listObjectsOfProperty(r,RDFS.label);
    if(ni.hasNext()){
      Literal l = (Literal)firstNode(ni);
      return l.getLexicalForm();
    }
    else
      return null;
  }

  /**
   * Add a resource as Class to the ontology.
   */
  private void addClass(Ontology ont, Resource r, String s, String comment
                        ,String label){
    ObjectId id = resourceToId(r);
    if(id==null)
      return;
    if(!id.getNameSpace().replace('#',' ').trim().equalsIgnoreCase(ont.getXMLBase()))
      return;  //pzx: 10/16/04: don't add foreign classes
    if(ont.getOntClass(id)==null){
      OntClass oc = ont.createClass();
      oc.setID(id);
      if(s.equalsIgnoreCase("deprecated"))
        oc.setDeprecated(true);
      if(comment!=null)
        oc.addDescription(comment);
      if(label!=null)
        oc.setLabel(label);
      ont.addOntClass(oc);
    }
  }

  /**
   * Add a resource as property.
   */
  private OntProperty addProperty(Ontology ont, Resource r, Model model, Boolean datatype){
    ObjectId id = resourceToId(r);
    if(id==null)
      return null;
    if(!r.getNameSpace().replace('#',' ').trim().equalsIgnoreCase(ont.getXMLBase()))
      return null;
    OntProperty op = ont.getProperty(id);
    if(op == null){
      op = ont.createProperty();
      op.setID(id);
      op.addDescription(getComment(r,model));
      op.setLabel(getLabel(r,model));
      if(datatype!=null)
        op.setDatatype(datatype.booleanValue());
      ont.addProperty(op);
    }
    return op;
  }


  /**
   * Wrap a resource into an ObjectId object.
   */
  private ObjectId resourceToId(Resource r){
    if(r==null)
      return null;
    if(!r.canAs(resource))
      return null;
    String s = r.toString();
    if(s==null)
      return null;
    if(r.isAnon())
      s = "["+s+"]";
    ObjectId id = new ObjectId(s);
    //id.setLocalName(r.getLocalName());
    //id.setNameSpace(r.getNameSpace());
    return id;
  }

  /**
   * OilEd: Count the number of elements in NodeIterator
   */
  private int count(NodeIterator ni){
    int c = 0;
    while (ni.hasNext()) {
	    ni.next();
	    c++;
    }
    return c;
  }

  /**
   * Combine two sets into one.
   */
  private Collection combineSets(Collection c1, Collection c2){
    ArrayList c = new ArrayList(c1);
    c.addAll(c2);
    return c;
  }

  private boolean isVocabularyResource(Resource r){
    if(r.equals(OWL.AllDifferent))
      return true;
    if(r.equals(OWL.Class))
      return true;
    if(r.equals(RDFS.Datatype))
      return true;
    if(r.equals(OWL.DatatypeProperty))
      return true;
    if(r.equals(OWL.DeprecatedClass))
      return true;
    if(r.equals(OWL.DeprecatedProperty))
      return true;
    if(r.equals(OWL.AnnotationProperty))
      return true;
    if(r.equals(OWL.FunctionalProperty))
      return true;
    if(r.equals(OWL.InverseFunctionalProperty))
      return true;
    if(r.equals(RDF.List))
      return true;
    if(r.equals(RDFS.Literal))
      return true;
    if(r.equals(RDF.nil))
      return true;
    if(r.equals(OWL.Nothing))
      return true;
    if(r.equals(OWL.ObjectProperty))
      return true;
    if(r.equals(OWL.Ontology))
      return true;
    if(r.equals(OWL.Restriction))
      return true;
    if(r.equals(OWL.SymmetricProperty))
      return true;
    if(r.equals(OWL.Thing))
      return true;
    if(r.equals(OWL.TransitiveProperty))
      return true;
    if(r.equals(RDF.Alt))
      return true;
    if(r.equals(RDF.Bag))
      return true;
    if(r.equals(RDF.List))
      return true;
    if(r.equals(RDF.nil))
      return true;
    if(r.equals(RDF.Seq))
      return true;
    if(r.equals(RDF.Property))
      return true;
    if(r.equals(RDF.Statement))
      return true;
    if(r.equals(RDFS.Class))
      return true;
    if(r.equals(RDFS.ConstraintProperty))
      return true;
    if(r.equals(RDFS.ConstraintResource))
      return true;
    if(r.equals(RDFS.Container))
      return true;
    if(r.equals(RDFS.ContainerMembershipProperty))
      return true;
    if(r.equals(RDFS.Datatype))
      return true;
    if(r.equals(RDFS.Literal))
      return true;
    if(r.equals(RDFS.Resource))
      return true;
    return false;
  }

  private StorageModel smodel;
  private Class resource = null; //to use REFNode.canAs(Class)

}
