/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/


package edu.lehigh.swat.hawk.owl;
//import edu.lehigh.swat.hawk.storage.OntWriter;
import edu.lehigh.swat.hawk.core.*;
import java.io.OutputStream;
import javax.xml.parsers.DocumentBuilder;
import org.apache.xerces.dom.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.LineSeparator;
import java.util.*;
import com.hp.hpl.jena.vocabulary.OWL;

/**
 * OWL writer, output OWL document in RDF format.
 * @author Z. Pan
 * @version 1.0
 */

public class OwlRdfWriter extends OntWriter{

  /**
   * Constructor
   */
  public OwlRdfWriter() {
  }

  /**
   * write a document to output
   */
  public void write(OutputStream out, edu.lehigh.swat.hawk.core.Document doc) throws Exception{
    output = out;
    xmlBase = doc.getXMLBase();
    ns = doc.getNameSpaces();

    document = new DocumentImpl();
    Element root = document.createElement("rdf:RDF");
    if(xmlBase!=null&&!xmlBase.equals(""))
      root.setAttribute("xml:base",xmlBase);
    document.appendChild(root);

    Element ontology = document.createElement("owl:Ontology");
    ontology.setAttribute("rdf:about","");
    //imports
    DocId[] imps = doc.getImports();
    if(imps!=null){
      int len = imps.length;
      for(int i=0;i<len;i++){
        Element eimp = document.createElement("owl:imports");
        eimp.setAttribute("rdf:resource",imps[i].getId());
        ontology.appendChild(eimp);
      }
    }

    Iterator it = doc.getAllDescriptions();
    while(it.hasNext()){
      String comment = (String)it.next();
      Element ecomment = document.createElement("rdfs:comment");
      Text txt = document.createTextNode(comment);
      ecomment.appendChild(txt);
      ontology.appendChild(ecomment);
    }

    String label = doc.getLabel();
    if(label!=null){
      Element elabel = document.createElement("rdfs:label");
      Text txt = document.createTextNode(label);
      elabel.appendChild(txt);
      ontology.appendChild(elabel);
    }

    root.appendChild(ontology);

    if(doc instanceof Ontology){
      Ontology ont = (Ontology)doc;

      DocId[] priors = ont.getPriorOnts();
      if(priors!=null){
        int len = priors.length;
        Element eimp = document.createElement("owl:priorVersion");
        for(int i=0;i<len;i++){
          Element eprior = document.createElement("owl:Ontology");
          eprior.setAttribute("rdf:about",priors[i].getId());
          eimp.appendChild(eprior);
        }
        ontology.appendChild(eimp);
      }

      DocId[] compatibles = ont.getCompatiblePriorOnts();
      if(compatibles!=null){
        int len = compatibles.length;
        Element eimp = document.createElement("owl:backwardCompatibleWith");
        for(int i=0;i<len;i++){
          eimp.setAttribute("rdf:resource",compatibles[i].getId());
        }
        ontology.appendChild(eimp);
      }

      OntClass[] classes = ont.getAllClasses();
      int len = classes.length;
      for(int i=0;i<len;i++){
        String name = classes[i].getID().getLocalName();
        if(name!=null&&!name.equals("")){
          Element c = writeClass(ont, classes[i]);
          root.appendChild(c);
        }
      }

      OntProperty[] properties = ont.getAllProperties();
      int plen = properties.length;
      for(int i=0;i<plen;i++){
        Element p = writeProperty(ont, properties[i]);
        root.appendChild(p);
      }
    } //end if ontology

    Individual[] individuals = doc.getAllIndividuals();
    int ilen = individuals.length;
    for(int i=0;i<ilen;i++){
      Element ind = writeIndividual(doc, individuals[i]);
      root.appendChild(ind);
    }

    //different
    Collection[] distinct = doc.getDistinctIndividualsSets();
    int dlen = distinct.length;
    for(int i=0;i<dlen;i++){
      if(distinct[i].size()<=2)
        continue;
      Element diff = document.createElement("owl:AllDifferent");
      Element dism = document.createElement("owl:distinctMembers");
      dism.setAttribute("rdf:parseType","Collection");
      Iterator ci = distinct[i].iterator();
      while(ci.hasNext()){
        ObjectId oid = (ObjectId)ci.next();
        Individual indiv = doc.getIndividual(oid);
        if(indiv==null)
          continue;
        ObjectId scid = indiv.getSourceClasses()[0];
        if(scid!=null){
          String prefix = getPrefix(scid.getNameSpace());
          Element dim = document.createElement(prefix+":"+scid.getLocalName());
          String name = applyBase(oid.getId());
          dim.setAttribute("rdf:about",name);
          dism.appendChild(dim);
        }

      }
      diff.appendChild(dism);
      root.appendChild(diff);
    }

    //nameSpaces
    Iterator nsi = ns.keySet().iterator();
    while(nsi.hasNext()){
      String uri = (String)nsi.next();
      String pre = (String)ns.get(uri);
      root.setAttribute("xmlns:"+pre,uri);
    }

    printOut();
  }


  /**
   * Create a element for an OntClass.
   */
  private Element writeClass(Ontology ont, OntClass ontClass){
    Element eclass;
    String cname = ontClass.getID().getLocalName();
    if(ontClass.isDeprecated()){
      eclass = document.createElement("owl:DeprecatedClass");
      if(cname!=null&&!cname.equals(""))
        eclass.setAttribute("rdf:ID",cname);
    }
    else{
      eclass = document.createElement("owl:Class");
      if(cname!=null&&!cname.equals(""))
        eclass.setAttribute("rdf:ID",cname);
    }
    //comment&label
    String comment = ontClass.getDescription();
    String label = ontClass.getLabel();
    if(comment!=null){
      Element ecomment = document.createElement("rdfs:comment");
      Text text = document.createTextNode(comment);
      ecomment.appendChild(text);
      eclass.appendChild(ecomment);
    }
    if(label!=null){
      Element elabel = document.createElement("rdfs:label");
      Text text = document.createTextNode(label);
      elabel.appendChild(text);
      eclass.appendChild(elabel);
    }

    //subs
    ObjectId[] sups = ontClass.getDirectSuperClasses();
    if(sups!=null){
      int len = sups.length;
      for(int i=0;i<len;i++){
        ObjectId id = sups[i];
        Element subc =
          document.createElement("rdfs:subClassOf");
        if(ont.getRestriction(id)==null){
          if(ont.getOntClass(id)!=null && id.getLocalName().equals("")){
            Element esuperc = writeClass(ont,ont.getOntClass(id));
            subc.appendChild(esuperc);
          }
          else{
          String name = applyBase(id.getId());
          subc.setAttribute("rdf:resource",name);
          }
        }
        else{
          Element r = writeRestriction(ont, ont.getRestriction(id));
          subc.appendChild(r);
        }
        eclass.appendChild(subc);
      }
    }

    ObjectId[] subs = ontClass.getDirectSubClasses();
    if(subs!=null){
      int len = subs.length;
      for(int i=0;i<len;i++){
        ObjectId id = subs[i];
        String ns = id.getNameSpace().replace('#',' ').trim();
        if(!ns.equalsIgnoreCase(ont.getXMLBase())){
          Element subc = document.createElement("owl:Class");
          subc.setAttribute("rdf:about", id.getId());
          Element subp = document.createElement("rdfs:subClassOf");
          subp.setAttribute("rdf:resource",applyBase(ontClass.getID().getId()));
          subc.appendChild(subp);
          document.getElementsByTagName("rdf:RDF").item(0).appendChild(subc);
        }
      }
    }

    //equivalent
    ObjectId[] equs = ontClass.getEquivalentClasses();
    if(equs!=null){
      int len = equs.length;
      for(int i=0;i<len;i++){
        ObjectId id = equs[i];
        Element equc =
          document.createElement("owl:equivalentClass");
        if(ont.getOntClass(id)!=null){
          String name = applyBase(id.getId());
          equc.setAttribute("rdf:resource",name);
        }
        else if(ont.getRestriction(id)!=null){
          Element r = writeRestriction(ont, ont.getRestriction(id));
          equc.appendChild(r);
        }
        eclass.appendChild(equc);
      }
    }
    //disjointWith
    ObjectId[] disjs = ontClass.getDisjointWith();
    if(disjs!=null){
      int len = disjs.length;
      for(int i=0;i<len;i++){
        ObjectId id = disjs[i];
        Element equc =
          document.createElement("owl:disjointWith");
        if(ont.getOntClass(id)!=null){
          String name = applyBase(id.getId());
          equc.setAttribute("rdf:resource",name);
        }
        else if(ont.getRestriction(id)!=null){
          Element r = writeRestriction(ont, ont.getRestriction(id));
          equc.appendChild(r);
        }
        eclass.appendChild(equc);
      }
    }
    //complementOf
    ObjectId[] comps = ontClass.getComplementOfs();
    if(comps!=null){
      int len = comps.length;
      Element comp =
          document.createElement("owl:complementOf");
         comp.setAttribute("rdf:parseType","Collection");
      if(len>0)
        eclass.appendChild(comp);
      for(int i=0;i<len;i++){
        ObjectId id = comps[i];
        if(ont.getOntClass(id)!=null){
          Element equc =
          document.createElement("owl:Class");
          String name = applyBase(id.getId());
          equc.setAttribute("rdf:about",name);
          comp.appendChild(equc);
        }
        else if(ont.getRestriction(id)!=null){
          Element r = writeRestriction(ont, ont.getRestriction(id));
          comp.appendChild(r);
        }
      }
    }
    //unionOf
    ObjectId[] unions = ontClass.getUnionOfs();
    if(unions!=null){
      int len = unions.length;
      Element union=
      document.createElement("owl:unionOf");
      union.setAttribute("rdf:parseType","Collection");
      if(len>0)
        eclass.appendChild(union);
      for(int i=0;i<len;i++){
        ObjectId id = unions[i];
        if(ont.getOntClass(id)!=null){
          Element equc =
          document.createElement("owl:Class");
          String name = applyBase(id.getId());
          equc.setAttribute("rdf:about",name);
          union.appendChild(equc);
        }
        else if(ont.getRestriction(id)!=null){
          Element r = writeRestriction(ont, ont.getRestriction(id));
          union.appendChild(r);
        }
      }
    }
    //intersectionOf
    ObjectId[] inters = ontClass.getIntersectionOfs();
    if(inters!=null){
      int len = inters.length;
      Element inter =
      document.createElement("owl:intersectionOf");
      inter.setAttribute("rdf:parseType","Collection");
      if(len>0)
        eclass.appendChild(inter);
      for(int i=0;i<len;i++){
        ObjectId id = inters[i];
        if(ont.getOntClass(id)!=null){
          Element equc =
          document.createElement("owl:Class");
          String name = applyBase(id.getId());
          equc.setAttribute("rdf:about",name);
          inter.appendChild(equc);
        }
        else if(ont.getRestriction(id)!=null){
          Element r = writeRestriction(ont, ont.getRestriction(id));
          inter.appendChild(r);
        }
      }
    }
    //oneOf
    ObjectId[] ones = ontClass.getOneOfs();
    if(ones!=null){
      int len = ones.length;
      Element one =
      document.createElement("owl:oneOf");
      one.setAttribute("rdf:parseType","Collection");
      if(len>0)
        eclass.appendChild(one);
      for(int i=0;i<len;i++){
        ObjectId id = ones[i];
          Element equc =
          document.createElement("owl:Thing");
          String name = applyBase(id.getId());
          equc.setAttribute("rdf:about",name);
          one.appendChild(equc);
      }
    }
    return eclass;
  }


  /**
   * Create an element for a Restriction.
   */
  private Element writeRestriction(Ontology ont, PropertyRestriction r){
    Element er = document.createElement("owl:Restriction");
    //onProperty
    ObjectId opid = r.getOnProperty();
    Element onProperty =
    document.createElement("owl:onProperty");
    String name = applyBase(opid.getId());
    onProperty.setAttribute("rdf:resource",name);
    er.appendChild(onProperty);
    //allValuesFrom
    ObjectId avf = r.getAllValuesFrom();
    if(avf!=null){
      Element eavf =
      document.createElement("owl:allValuesFrom");
      name = avf.getLocalName();
      if(name==null||name.equals("")){
        if(ont.getOntClass(avf)!=null){
          Element ecc = writeClass(ont,ont.getOntClass(avf));
          eavf.appendChild(ecc);
        }
      }
      else
        eavf.setAttribute("rdf:resource",applyBase(avf.getId()));
      er.appendChild(eavf);
    }
    //someValuesFrom
    ObjectId svf = r.getSomeValuesFrom();
    if(svf!=null){
      Element esvf =
      document.createElement("owl:someValuesFrom");
      name = svf.getLocalName();
      if(name==null||name.equals("")){
        if(ont.getOntClass(svf)!=null){
          Element ecc = writeClass(ont,ont.getOntClass(svf));
          esvf.appendChild(ecc);
        }
      }
      else
        esvf.setAttribute("rdf:resource",applyBase(svf.getId()));
      er.appendChild(esvf);
    }
    //hasValue
    ObjectId hv = r.getHasValue();
    if(hv!=null){
      if(hv instanceof DatatypeValue){
        DatatypeValue dv = (DatatypeValue)hv;
        Element ehv =
        document.createElement("owl:hasValue");
        ehv.setAttribute("rdf:datatype",dv.getType());
        ehv.setNodeValue(dv.getValue());
        er.appendChild(ehv);
      }
      else{
        Element ehv =
        document.createElement("owl:hasValue");
        name = applyBase(hv.getId());
        ehv.setAttribute("rdf:resource",name);
        er.appendChild(ehv);
      }
    }
    //cardinality
    int maxcard = r.getMaxCardinality();
    int mincard = r.getMinCardinality();

    if(mincard==maxcard){
      Element ecard =
      document.createElement("owl:cardinality");
      ecard.setAttribute("rdf:datatype","http://www.w3.org/2001/XMLSchema#nonNegativeInteger");
      Text text = document.createTextNode(String.valueOf(mincard));
      ecard.appendChild(text);
      er.appendChild(ecard);
    }
    //maxCardinality

    if(maxcard!=PropertyRestriction.INFINIT&&maxcard!=mincard){
      Element emaxcard =
      document.createElement("owl:maxCardinality");
      emaxcard.setAttribute("rdf:datatype","http://www.w3.org/2001/XMLSchema#nonNegativeInteger");
      Text text = document.createTextNode(String.valueOf(maxcard));
      emaxcard.appendChild(text);
      er.appendChild(emaxcard);
    }
    //minCardinality

    if(mincard!=0&&mincard!=maxcard){
      Element emincard =
      document.createElement("owl:minCardinality");
      emincard.setAttribute("rdf:datatype","http://www.w3.org/2001/XMLSchema#nonNegativeInteger");
      Text text = document.createTextNode(String.valueOf(mincard));
      emincard.appendChild(text);
      er.appendChild(emincard);
    }
    return er;
  }

  /**
   * Create an element for a OntProperty.
   */
  private Element writeProperty(Ontology ont, OntProperty property){
    Element eproperty = null;
    if(property.isDatatype()!=null&&property.isDatatype().booleanValue())
      eproperty = document.createElement("owl:DatatypeProperty");
    if(property.isDatatype()!=null&&!property.isDatatype().booleanValue())
      eproperty = document.createElement("owl:ObjectProperty");
    if(property.isDatatype()==null)
      eproperty = document.createElement("rdf:Property");
    if(property.isAnnotation())
      eproperty = document.createElement("owl:AnnotationProperty");

    String name = property.getID().getLocalName();
    eproperty.setAttribute("rdf:ID",name);
    if(property.isDeprecated()){
      Element eptype = document.createElement("rdf:type");
      eptype.setAttribute("rdf:resource", OWL.DeprecatedProperty.getURI());
      eproperty.appendChild(eptype);
    }
    if(property.isFunctional()){
      Element eptype = document.createElement("rdf:type");
      eptype.setAttribute("rdf:resource", OWL.FunctionalProperty.getURI());
      eproperty.appendChild(eptype);
    }

    if(property.isInverseFunctional()){
      Element eptype = document.createElement("rdf:type");
      eptype.setAttribute("rdf:resource", OWL.InverseFunctionalProperty.getURI());
      eproperty.appendChild(eptype);
    }

    if(property.isSymmetric()){
      Element eptype = document.createElement("rdf:type");
      eptype.setAttribute("rdf:resource", OWL.SymmetricProperty.getURI());
      eproperty.appendChild(eptype);
    }

    if(property.isTransitive()){
      Element eptype = document.createElement("rdf:type");
      eptype.setAttribute("rdf:resource", OWL.TransitiveProperty.getURI());
      eproperty.appendChild(eptype);
    }



    //comment&label
    String comment = property.getDescription();
    String label = property.getLabel();
    if(comment!=null){
      Element ecomment = document.createElement("rdfs:comment");
      Text text = document.createTextNode(comment);
      ecomment.appendChild(text);
      eproperty.appendChild(ecomment);
    }
    if(label!=null){
      Element elabel = document.createElement("rdfs:label");
      Text text = document.createTextNode(label);
      elabel.appendChild(text);
      eproperty.appendChild(elabel);
    }

    ObjectId[] subs = property.getDirectSubProperties();
    if(subs!=null){
      int len = subs.length;
      for(int i=0;i<len;i++){
        ObjectId id = subs[i];
        String ns = id.getNameSpace().replace('#',' ').trim();
        if(!ns.equalsIgnoreCase(ont.getXMLBase())){
          Element subc = document.createElement("rdf:Property");
          subc.setAttribute("rdf:about", id.getId());
          Element subp = document.createElement("rdfs:subPropertyOf");
          subp.setAttribute("rdf:resource",applyBase(property.getID().getId()));
          subc.appendChild(subp);
          document.getElementsByTagName("rdf:RDF").item(0).appendChild(subc);
        }
      }
    }

    ObjectId[] sups = property.getDirectSuperProperties();
    if(subs!=null){
      int len = sups.length;
      for(int i=0;i<len;i++){
        ObjectId id = sups[i];
        Element sup =
        document.createElement("rdfs:subPropertyOf");
        name = applyBase(id.getId());
        sup.setAttribute("rdf:resource",name);
        eproperty.appendChild(sup);
      }
    }

    ObjectId[] equs = property.getEquivalentProperties();
    if(equs!=null){
      int len = equs.length;
      for(int i=0;i<len;i++){
        ObjectId id = equs[i];
        Element equ =
        document.createElement("owl:equivalentProperty");
        name = applyBase(id.getId());
        equ.setAttribute("rdf:resource",name);
        eproperty.appendChild(equ);
      }
    }

    ObjectId[] invs = property.getInverseOf();
    if(invs!=null){
      int len = invs.length;
      for(int i=0;i<len;i++){
        ObjectId id = invs[i];
        Element inv =
        document.createElement("owl:inverseOf");
        name = applyBase(id.getId());
        inv.setAttribute("rdf:resource",name);
        eproperty.appendChild(inv);
      }
    }

    ObjectId domain = property.getDomain();
    if(domain!=null){
      Element edomain =
      document.createElement("rdfs:domain");
      name = applyBase(domain.getId());
      edomain.setAttribute("rdf:resource",name);
      eproperty.appendChild(edomain);
    }

    ObjectId range = property.getRange();
    if(range!=null){
      Element erange =
      document.createElement("rdfs:range");
      name = applyBase(range.getId());
      erange.setAttribute("rdf:resource",name);
      eproperty.appendChild(erange);
    }

    return eproperty;
  }

  /**
   * Create an element for an individual.
   */
  private Element writeIndividual(edu.lehigh.swat.hawk.core.Document doc, Individual indiv){
    ObjectId[] source = indiv.getSourceClasses();
    int slen = source.length;
    String prefix = getPrefix(source[0].getNameSpace());
    String sourceName = "";
    if(prefix.equals(""))
      sourceName = source[0].getLocalName();
    else
      sourceName = prefix+":"+source[0].getLocalName();
    Element eindiv = document.createElement(sourceName);
    ObjectId iid = indiv.getID();
    String name = applyBase(iid.getId());
    if(!name.equals(""))
      eindiv.setAttribute("rdf:about",name);

    if(slen>1){
      for(int i=1;i<slen;i++){
        Element etype = document.createElement("rdf:type");
        etype.setAttribute("rdf:resource",source[i].getId());
        eindiv.appendChild(etype);
      }
    }
    //comment&label
    String comment = indiv.getDescription();
    String label = indiv.getLabel();
    if(comment!=null){
      Element ecomment = document.createElement("rdfs:comment");
      Text text = document.createTextNode(comment);
      ecomment.appendChild(text);
      eindiv.appendChild(ecomment);
    }
    if(label!=null){
      Element elabel = document.createElement("rdfs:label");
      Text text = document.createTextNode(label);
      elabel.appendChild(text);
      eindiv.appendChild(elabel);
    }
    //sameAs&differentFrom
    ObjectId[] sames = indiv.getAllSameAs();
    if(sames!=null){
      int len = sames.length;
      for(int i=0;i<len;i++){
        ObjectId oid = sames[i];
        Element esame = document.createElement("owl:sameIndividualAs");
        esame.setAttribute("rdf:resource",applyBase(oid.getId()));
        eindiv.appendChild(esame);
      }
    }

    ObjectId[] diffs = indiv.getAllDifferentFrom();
    if(diffs!=null){
      int len = diffs.length;
      for(int i=0;i<len;i++){
        ObjectId oid = diffs[i];
        Element ediff = document.createElement("owl:differentFrom");
        ediff.setAttribute("rdf:resource",applyBase(oid.getId()));
        eindiv.appendChild(ediff);
      }
    }

    ObjectId[] pids = indiv.getProperties();
    if(pids!=null){
      int len = pids.length;
      for(int i=0;i<len;i++){
        ObjectId pid = pids[i];
        ObjectId[] values = indiv.getValuesForProperty(pid);
        if(values!=null){
          int vlen = values.length;
          prefix = getPrefix(pid.getNameSpace());
          String pName = "";
          if(prefix.equals(""))
            pName = pid.getLocalName();
          else
            pName = prefix+":"+pid.getLocalName();
          for(int j=0;j<vlen;j++){
            Element epv = document.createElement(pName);
            if(values[j] instanceof DatatypeValue){
              DatatypeValue dv = (DatatypeValue)values[j];
              if(dv.getType()!=null)
                epv.setAttribute("rdf:datatype",dv.getType());
              Text text = document.createTextNode(dv.getValue());
              epv.appendChild(text);
            }else{
              name = applyBase(values[j].getId());
              epv.setAttribute("rdf:resource", name);
            }
            eindiv.appendChild(epv);

          }
        }
      }
    }
    return eindiv;
  }

  /**
   * Print out the DOM tree.
   */
  private void printOut(){
    try{
    OutputFormat format = new OutputFormat(document);
    format.setIndent(2);
    format.setLineSeparator(System.getProperty("line.separator"));
    format.setLineWidth(80);

    //format.setPreserveSpace(true);
    XMLSerializer serializer = new XMLSerializer (
        output, format);
    serializer.asDOMSerializer();
    serializer.serialize(document);
    }
    catch(Exception e){
      System.err.println(e.getMessage());
    }
  }

  /**
   * Get the prefix for nameSpace.
   */
  private String getPrefix(String uri){
    if(ns.containsKey(uri))
      return (String)ns.get(uri);
    else{
      String pre = "ns"+nsCount;
      nsCount++;
      ns.put(uri,pre);
      return pre;
    }
  }

  /**
   * Apply xml:base to shorten uri.
   */
  private String applyBase(String uri){
    if(uri==null||uri.equals(""))
      return uri;
    int p = uri.indexOf('#');
    if(p>0){
      String base = uri.substring(0,p);
      if(base.equalsIgnoreCase(xmlBase))
        return uri.substring(p,uri.length());
    }
    return uri;
  }

  private OutputStream output;
  private org.w3c.dom.Document document;
  private String xmlBase;
  private HashMap ns;
  private int nsCount = 0;


}
