/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.dlmemory;

import org.apache.xmlbeans.*;
import java.io.*;
import java.net.*;
import edu.lehigh.swat.hawk.core.*;
import edu.lehigh.swat.hawk.owl.OwlDigWriter;
import org.kr.dl.dig.v1_1.*;
import org.kr.dl.dig.reasoner.v1_1.Reasoner;
import org.kr.dl.dig.reasoner.v1_1.impl.HTTPReasoner;


/**
 * An implementation of core.ontology, can do inference with a DL reasoner.
 * @author Z. Pan
 * @version 1.0
 */
public class Ontology extends edu.lehigh.swat.hawk.storage.simplememory.Ontology{

  /**
   * Constructor.
   */
  public Ontology(){
    super();
  }


  /**Use a DL reasoner to do inference on this ontology, then rearrange it based upon
   * the results. Currently we check the satisfibility and ask for parents of each
   * class. We also asks for the equivalent classes of each class.
   * Note: we don't delete the restrictions in the original superClass set.
   * Thus in one class's superClass set, it's possible that some property restrictions subsume one of the direct
   * superClass.
   * @param rUrl The url of the http reasoner, by default it is http://localhost:8080 .
   */
  public void computeSubsumptions(String rUrl, edu.lehigh.swat.hawk.core.Ontology[] imports){
    try{
      //System.setProperty("dig.shiqapp","D:\\OWLparser\\OWLParserreasoner\\shiq-app.exe");
      if(rUrl==null||rUrl.equals(""))
        rUrl = "http://localhost:8080";
      URL reasonerUrl = new URL(rUrl);
      Reasoner r = new HTTPReasoner(reasonerUrl);

      URI kburi = r.newKB();
      OwlDigWriter writer = new OwlDigWriter();
      StringWriter sw = new StringWriter();
      writer.writeOntology(this,sw);
      sw.flush();
      /* //test
      PrintStream ps = new PrintStream(new FileOutputStream("d:\\digout.xml"));
      ps.print(sw.toString());
      ps.close();
      */ //end test
      TellsDocument td = TellsDocument.Factory.parse(sw.toString());
      ResponseDocument rd = r.tells(td);
      IdRespType rt = rd.getResponse();
      if(rt.isSetOk())
        System.out.println("tells OK");
      else if(rt.isSetError()){
        System.out.println("tells error");
        ErrorDocument.Error er = rt.getError();
        System.out.println(er.getMessage());
        return;
      }

      if(imports!=null){
        int len = imports.length;
        for(int i=0;i<len;i++){
          edu.lehigh.swat.hawk.core.Ontology ionto = imports[i];
          StringWriter isw = new StringWriter();
          writer.writeOntology(ionto,isw);
          isw.flush();
          TellsDocument itd = TellsDocument.Factory.parse(isw.toString());
          ResponseDocument ird = r.tells(itd);
          IdRespType irt = ird.getResponse();
          if(irt.isSetOk())
            System.out.println("tells OK");
          else if(irt.isSetError()){
            System.out.println("tells error");
            ErrorDocument.Error er = irt.getError();
            System.out.println(er.getMessage());
          }
        }
     }


      AsksDocument askDoc = AsksDocument.Factory.newInstance();
      Ask askSat = askDoc.addNewAsks();
      OntClass[] classes = this.getAllClasses();
      int len =0;
      if(classes!=null)
        len = classes.length;
      for(int i=0;i<len;i++){
        OntClass cls = classes[i];
        String cid = cls.getID().getId();
        ConceptId conid = askSat.addNewSatisfiable();
        conid.setId("q"+i);
        Named named = conid.addNewCatom();
        named.setName(cid);
      }

      ResponsesDocument rSat = r.asks(askDoc);
      ResponseType rtype = rSat.getResponses();
      IdType[] ftypes = rtype.getFalseArray();
      int flen = ftypes.length;
      for(int i=0;i<flen;i++){
        String seq = ftypes[i].getId().substring(1);
        int rseq = Integer.parseInt(seq);
        classes[rseq].setSatisfiable(new Boolean(false));
      }
      IdType[] ttypes = rtype.getTrueArray();
      int tlen = ttypes.length;
      for(int i=0;i<tlen;i++){
        String seq = ttypes[i].getId().substring(1);
        int rseq = Integer.parseInt(seq);
        classes[rseq].setSatisfiable(new Boolean(true));
      }

      AsksDocument askDoc2 = AsksDocument.Factory.newInstance();
      Ask askSup = askDoc2.addNewAsks();
      for(int i=0;i<len;i++){
        OntClass cls = classes[i];
        String cid = cls.getID().getId();
        ConceptId cpid = askSup.addNewParents();
        cpid.setId("p"+i);
        ConceptId ceid = askSup.addNewEquivalents();
        ceid.setId("e"+i);
        Named named = cpid.addNewCatom();
        named.setName(cid);
        Named named2 = ceid.addNewCatom();
        named2.setName(cid);
      }

      ResponsesDocument rsup = r.asks(askDoc2);
      ResponseType stype = rsup.getResponses();
      Csynonyms[] csys = stype.getConceptSetArray();
      int clen = csys.length;
      for(int i=0;i<clen;i++){
        Csynonyms csy = csys[i];
        String rid = csy.getId();
        String seq = rid.substring(1);
        int rseq = Integer.parseInt(seq);

        Concepts[] ca = csy.getSynonymsArray();
        int sylen = ca.length;
        if(rid.startsWith("p")){
          emptySupers(classes[rseq]);
          for(int j=0;j<sylen;j++){
            Named[] cnames = ca[j].getCatomArray();
            int nlen = cnames.length;
            for(int k=0;k<nlen;k++){
              ObjectId newSuper = new ObjectId(cnames[k].getName());
              classes[rseq].addSuperClass(newSuper);
            }
          }
        }
        if(rid.startsWith("e")){
          for(int j=0;j<sylen;j++){
            Named[] cnames = ca[j].getCatomArray();
            int nlen = cnames.length;
            for(int k=0;k<nlen;k++){
              ObjectId newEqu = new ObjectId(cnames[k].getName());
              classes[rseq].addEquivalentClass(newEqu);
            }
          }
        }
      }

//properties
      OntProperty[] properties = this.getAllProperties();
      int plen =0;
      if(properties!=null)
         plen = properties.length;
      AsksDocument askDoc3 = AsksDocument.Factory.newInstance();
      Ask askSupp = askDoc3.addNewAsks();
      for(int i=0;i<plen;i++){
        OntProperty prop = properties[i];
        String cid = prop.getID().getId();
        RoleId cpid = askSupp.addNewRparents();
        cpid.setId("pp"+i);
//        ConceptId ceid = askSupp.addNewRequivalents();
//        ceid.setId("pe"+i);
        Named named = cpid.addNewRatom();
        named.setName(cid);
//        Named named2 = ceid.addNewRatom();
 //       named2.setName(cid);
      }

      ResponsesDocument rsupp = r.asks(askDoc3);
      ResponseType pstype = rsupp.getResponses();
      Rsynonyms[] pcsys = pstype.getRoleSetArray();
      int pclen = pcsys.length;
      for(int i=0;i<pclen;i++){
        Rsynonyms csy = pcsys[i];
        String rid = csy.getId();
        String seq = rid.substring(2);
        int rseq = Integer.parseInt(seq);

        Roles[] ca = csy.getSynonymsArray();
        int sylen = ca.length;
        if(rid.startsWith("pp")){
          emptySupers(properties[rseq]);
          for(int j=0;j<sylen;j++){
            Named[] cnames = ca[j].getRatomArray();
            int nlen = cnames.length;
            for(int k=0;k<nlen;k++){
              ObjectId newSuper = new ObjectId(cnames[k].getName());
              properties[rseq].addSuperProperty(newSuper);
            }
          }
        }
      }


     r.releaseKB(kburi);
    }
    catch(Exception e){
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }

  /**
   * Empty the superClasses.
   * @param c
   */
  private void emptySupers(OntObject x){
    if(x instanceof OntClass){
      OntClass c = (OntClass)x;
      ObjectId[] supers = c.getDirectSuperClasses();
      int len = supers.length;
      for(int i=0;i<len;i++){
        if(!supers[i].getLocalName().equals("")) //z.pan: be careful here.
          c.deleteSuperClass(supers[i]);
      }
    }
    if(x instanceof OntProperty){
      OntProperty p = (OntProperty)x;
      ObjectId[] supers = p.getDirectSuperProperties();
      int len = supers.length;
      for(int i=0;i<len;i++){
        if(!supers[i].getLocalName().equals("")) //z.pan: be careful here.
          p.deleteSuperProperty(supers[i]);
      }
    }
  }

}
