/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.simpledb;

/**
 * Database storage model.
 * @author Z. Pan
 * @version 1.0
 */

import edu.lehigh.swat.hawk.storage.*;
import java.sql.*;
import java.util.*;
import java.io.*;
import edu.lehigh.swat.hawk.core.ObjectId;
import edu.lehigh.swat.hawk.core.DocId;


public class DatabaseStorage extends PersistentStorageModel implements QueryableStorageModel{

  /**
   * Constructor.
   * @param DBName
   */
  public DatabaseStorage(String DBName) {
    dbName = DBName;
    queryOnto = "";
  }

  /**
   * Open a connection to the database.
   */
  public void open(){
    String db_prefix = "";
    try{
      if(dbName.endsWith(".mdb")){
        java.lang.Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
        db_prefix = "jdbc:odbc:MS Access Database;DBQ=";
        emptyTableState = "S0002";
      }
      else if(dbName.indexOf("postgresql")>0){
        java.lang.Class.forName("org.postgresql.Driver");
        emptyTableState = "42P01";
        System.out.println("postgresql!");
      }
     }
    catch(java.lang.ClassNotFoundException e){
      System.out.print("ClassNotFoundException: ");
      System.out.println(e.getMessage());
    }

    try{
      String[] names = dbName.split("@");
      String[] login = names[0].split(":",2);
      String s_open = db_prefix+names[1];
      System.out.println(s_open);
      con = java.sql.DriverManager.getConnection(s_open,login[0],login[1]);
    }
    catch(SQLException e){
      System.out.println("error when connecting database");
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
      stmt = con.createStatement(java.sql.ResultSet.TYPE_FORWARD_ONLY,java.sql.ResultSet.CONCUR_READ_ONLY);
     }
    catch(SQLException e){
      System.out.println("error_in_stmt");
    }

    knownOntos = getKnownOntologies();
    getSourceNum();
    getKnownUrls();
    cachedDocs = new Hashtable();
    loadedDocs = new Hashtable();
  }

  /**
   * Close the database connection.
   */
  public void close(){
    Iterator it = loadedDocs.values().iterator();
    while(it.hasNext()){
      Document doc = (Document)it.next();
 //     doc.flush();
      if(doc instanceof Ontology && serializeOnto){
        Ontology ontd = (Ontology)doc;
        int slash = dbName.lastIndexOf('\\');
        int sat = dbName.indexOf('@');
        String dbpath = "";
        if(slash>0){
          dbpath = dbName.substring(sat+1,slash+1);
        }
        String sname = dbpath+"onto"+ontd.getOntSeq()+".dat";
        try{
          FileOutputStream out = new FileOutputStream(sname);
          ObjectOutputStream s = new ObjectOutputStream(out);
          edu.lehigh.swat.hawk.storage.simplememory.Ontology monto = ontd.getMemOnto();
          s.writeObject(monto);
          s.flush();

          s.close();
          out.close();
        }
        catch(Exception e){
          System.err.println(e.getMessage());
          e.printStackTrace();
        }
      }
    }
    try{
      stmt.close();
      con.close();
      }
    catch(SQLException e){
      System.out.println("error when close kb");
    }
  }


  /**
   * Add a document to this model.
   */
  public void addDocument(edu.lehigh.swat.hawk.core.Document d){
    if(d.getID()==null){
      System.out.println("Error: ID not found in a document");
      return;
    }
    loadedDocs.remove(d.getID());
    loadedDocs.put(d.getID(),d);
    if(d instanceof Ontology){
      Ontology ontd = (Ontology)d;
      updateOntIndex(ontd, ontd.getOntSeq());
      updateSourceIndex(ontd, ontd.getDocSeq());
    }
    else{
      Document docd = (Document)d;
      updateSourceIndex(docd, docd.getDocSeq());
    }
  }

  /**
   * Get the specified document.
   */
  public edu.lehigh.swat.hawk.core.Document getDocument(String url){
    Document doc = null;
    DocId did = new DocId(url);
    if(loadedDocs.containsKey(did))
      return (edu.lehigh.swat.hawk.core.Document)loadedDocs.get(did);
    if(cachedDocs.containsKey(did))
      return (edu.lehigh.swat.hawk.core.Document)cachedDocs.get(did);
    if(knownOntos.containsKey(url)){
       Integer i = (Integer)knownOntos.get(url);
       int seq = i.intValue();
       int slash = dbName.lastIndexOf('\\');
       String dbpath = "";
       if(slash>0){
        dbpath = dbName.substring(0,slash+1);
       }
       String sname = dbpath+"onto"+seq+".dat";
       try{
         FileInputStream ins = new FileInputStream(sname);
         ObjectInputStream ois = new ObjectInputStream(ins);
         edu.lehigh.swat.hawk.storage.simplememory.Ontology mOnto =
             (edu.lehigh.swat.hawk.storage.simplememory.Ontology) ois.readObject();
         doc = new Ontology(mOnto);
         doc.setID(mOnto.getID());
         cachedDocs.put(doc.getID(), doc);
       }
       catch(Exception e){
         System.err.println(e.getMessage());
       }return doc;
    }
    if(knownDocs.containsKey(url)){
      doc = new Document();
      int seq = ((Integer)knownDocs.get(url)).intValue();
      doc.setDocSeq(seq);
      return doc;
    }
    return null;
  }


  /**
   * Update a document.
   */
  public void updateDocument(edu.lehigh.swat.hawk.core.Document d){

  }

  /**
   * Get all the documents in the model.
   */
  public Iterator getDocumentIterator(){
    return null;
  }

  /**
   * Set whether or not include implictly imported documents.
   */
  public void setImplictImports(boolean imp){

  }


  /**
   * Create an Ontology object.
   */
  public edu.lehigh.swat.hawk.core.Ontology createOntology(){
    edu.lehigh.swat.hawk.storage.simplememory.Ontology mOnto =
        new edu.lehigh.swat.hawk.storage.simplememory.Ontology();
    Ontology onto = new Ontology(mOnto);
    onto.setDocSeq(++sourceSeqNum);
    onto.setOntSeq(++ontSeqNum);
    onto.setSQLStatement(stmt);
    onto.setModel(this);
    return onto;
  }

  /**
   * Create an Document object.
   */
  public edu.lehigh.swat.hawk.core.Document createDocument(){
    Document doc = new Document();
    doc.setDocSeq(++sourceSeqNum);
    doc.setSQLStatement(stmt);
    doc.setModel(this);
    return doc;
  }

  /**
   *retrieval ontologies already exist in database
   */
  public Hashtable getKnownOntologies()
  {
    int seqNum = 0;
    String ontoUrl;
    String qs = "SELECT * \nFROM ontologies_index";
    String qs2 = "SELECT MAX(seqnum) FROM ontologies_index";
    Hashtable knownOntologies = new Hashtable();
    String errState = "";
    java.sql.ResultSet rs = null;

    try{
      rs = stmt.executeQuery(qs);
     }
    catch(SQLException e){
      //errcode = e.getErrorCode();
     // if(errcode!=-1305){
       errState = e.getSQLState();
       if(errState.equals(emptyTableState)){
         createKb();
         ontSeqNum = 0;
         return knownOntologies;
       }else{
        System.out.println("error");
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
      }
    }

    try{
      while (rs.next()){
        ontoUrl = rs.getString("url");
        seqNum = rs.getInt("seqnum");
        knownOntologies.put(ontoUrl, new Integer(seqNum));
      }
      rs.close();
      rs = stmt.executeQuery(qs2);
      if(rs.next())
        ontSeqNum = rs.getInt(1);
      rs.close();
    }

    catch (SQLException e){
        System.out.println(e.getMessage());
     }

    knownOntos = knownOntologies;
    return knownOntologies;
  }


  /**
   * Get the maximum source sequence number in the databade.
   */
  public void getSourceNum()
  {
    knownDocs = new Hashtable();
    int seqNum = 0;
    String qs = "SELECT * \nFROM source_index";
    String qs2 = "SELECT MAX(seqnum) FROM source_index";
    String errState = "";
    java.sql.ResultSet rs = null;
    try{
      rs = stmt.executeQuery(qs);
     }
    catch(SQLException e){
      System.out.println("error");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
      errState = e.getSQLState();
    }
    if (errState.equals(emptyTableState)){
          createKb();
          sourceSeqNum = 0;
          return;
    }
    try{
      while (rs.next()){
        String url = rs.getString("url");
        seqNum = rs.getInt("seqnum");
        knownDocs.put(url,new Integer(seqNum));
      }
      rs.close();
      rs = stmt.executeQuery(qs2);
      if(rs.next())
        sourceSeqNum = rs.getInt(1);
      rs.close();
    }catch (SQLException e){
        System.out.println(e.getMessage());
    }
  }


  /**
   * Initialize the database. Create the index tables and utility tables.
   */
  public void createKb(){
    try{
      stmt.executeUpdate("CREATE TABLE Ontologies_index ("+
                               "url VARCHAR (150),"+
                               "SEQNUM INTEGER,"+
                               "PRIMARY KEY (SEQNUM)"+
                               ")");
    } catch(SQLException e){
            System.out.println("error when creat table 'Ontologies_index'");
            System.out.println("SQLState:"+e.getSQLState());
            System.out.println("Message :"+e.getMessage());
            System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
    stmt.executeUpdate("CREATE TABLE source_index ("+
                               "url VARCHAR (150),"+
                               "SEQNUM INTEGER,"+
                               "PRIMARY KEY (SEQNUM)"+
                               ")");
    }catch(SQLException e){
      System.out.println("error when creat table 'source_index'");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
      stmt.executeUpdate("CREATE TABLE url_index ("+
                         "url VARCHAR (200),"+
                         "ID INTEGER,"+
                         "PRIMARY KEY (ID)"+
                               ")");
    }catch(SQLException e){
      System.out.println("error when creat table 'url_index'");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
      stmt.executeUpdate("CREATE index urlIndex ON url_index (url)");
    } catch(SQLException e){
      System.out.println("error when index url");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
      stmt.executeUpdate("CREATE TABLE propertytypes ("+
                                  "pname VARCHAR (100),"+
                                  "PRIMARY KEY (pname),"+
                                  "type VARCHAR (20) );");
    }catch(SQLException e){
      System.out.println("error when create propertytype table");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
    stmt.executeUpdate("CREATE TABLE thing_owl ("+
                       "ID VARCHAR (200),"+
                       "PRIMARY KEY (ID),"+
                       "source INTEGER);");
    }catch(SQLException e){
     System.out.println("error when create propertytype table");
     System.out.println("SQLState:"+e.getSQLState());
     System.out.println("Message :"+e.getMessage());
     System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
    if(dbName.endsWith(".mdb")){
      stmt.executeUpdate("CREATE TABLE view_definition ("+
                       "view VARCHAR (100),"+
                       "PRIMARY KEY (view),"+
           "definition MEMO);");

    }else
      stmt.executeUpdate("CREATE TABLE view_definition ("+
                       "view VARCHAR (100),"+
                       "PRIMARY KEY (view),"+
           "definition TEXT);");
    }catch(SQLException e){
     System.out.println("error when create view_definitions table");
     System.out.println("SQLState:"+e.getSQLState());
     System.out.println("Message :"+e.getMessage());
     System.out.println("Vendor  :"+e.getErrorCode());
    }
  }

  /**
   * Get the table name for giving id.
   */
  public String getTableName(ObjectId id){
    String local = id.getLocalName();
    String ontoName = id.getNameSpace();
    ontoName = ontoName.replace('#',' ').trim();
/*
    Document doc = (Document)getDocument(ontoName);

    if(doc==null||!(doc instanceof Ontology)){
      System.out.println("+++ Can not find onto that defined term: "+id.getId());
      return null;
    }
    Ontology ont = (Ontology)doc;
    if(!ont.hasClass(local)&&!ont.hasProperty(local)){
      return null;
    }
*/ //pan on 10/28/05
    Integer seq = (Integer)knownOntos.get(ontoName);
    if(seq!=null)
      return local.replace('-','_')+"_"+seq.intValue();
    else if(local.equalsIgnoreCase("Thing")&&ontoName.startsWith("http://www.w3.org/2002/07/owl"))
      return "Thing_OWL";
    return null;
  }

  /**
   * Retrieval Urls already exist in database
   */
  public Hashtable getKnownUrls()
  {
    java.sql.ResultSet rs = null;
    int id = 0;
    String url;
    String qs = "SELECT MAX(id) FROM url_index";
    knownUrls = new Hashtable();

    try{
      rs = stmt.executeQuery(qs);
     }
    catch(SQLException e){
      System.out.println("error");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
    }

    try{
      if(rs.next()){
        id = rs.getInt(1);
      }
      rs.close();
      maxUrlNum = id;
    }
    catch (SQLException e){
      System.out.println(e.getMessage());
    }

    return knownUrls;
  }


  /**
   * Add the new ontology to index.
   */
  private void updateOntIndex(Ontology onto, int seqnum)
  {

     String qs = "INSERT INTO ONTOLOGIES_INDEX VALUES ('"+onto.getID().getId()+"', "
                   +String.valueOf(seqnum)+")";
     try {
     stmt.executeUpdate(qs);
     }
     catch(SQLException e)
               {
                 System.out.println("error when update onto index");
                 System.out.println(qs);
                 System.out.println("SQLState:"+e.getSQLState());
                 System.out.println("Message :"+e.getMessage());
                 System.out.println("Vendor  :"+e.getErrorCode());
               }
     knownOntos.put(onto.getID().getId(), new Integer(seqnum));
  }

  /**
   * Add a new document to the index.
   */
  private void updateSourceIndex(Document onto, int seqnum)
  {

     String qs = "INSERT INTO SOURCE_INDEX VALUES ('"+onto.getID().getId()+"', "
                   +String.valueOf(seqnum)+")";
   //  log.println(qs);
     try {
     stmt.executeUpdate(qs);
     }
     catch(SQLException e)
               {
                 System.out.println("error when update source index");
                 System.out.println(qs);
                 System.out.println("SQLState:"+e.getSQLState());
                 System.out.println("Message :"+e.getMessage());
                 System.out.println("Vendor  :"+e.getErrorCode());

               }
  }

  /**
   * Get the index for given uri.
   */
  protected int getUrlIndex(String url){
    int urlId = -1;
    if(knownUrls.get(url)!=null){
      urlId = ( (Integer) knownUrls.get(url)).intValue();
    }
    else{
      String qus = "SELECT id FROM url_index WHERE url='"+url+"'";
      try{
        java.sql.ResultSet rs = stmt.executeQuery(qus);
        if(rs.next())  {
          urlId = rs.getInt(1);
          knownUrls.put(url,new Integer(urlId));
        }
        else  {
          urlId = addUrl(url);
        }
      }
      catch(SQLException e){
                  System.out.println("error");
                  System.out.println("SQLState:"+e.getSQLState());
                  System.out.println("Message :"+e.getMessage());
                  System.out.println("Vendor  :"+e.getErrorCode());
      }
    }
    return urlId;
  }

  /**
   * Add a new url to the url index.
   */
  private int addUrl(String url)
  {
    String qs = "INSERT INTO URL_INDEX VALUES ('"+url+"', "
                   +String.valueOf(++maxUrlNum)+")";
     try {
     stmt.executeUpdate(qs);
     knownUrls.put(url,new Integer(maxUrlNum));
     return maxUrlNum;
     }
     catch(SQLException e)
               {
                 System.out.println("error when update source index");
                 System.out.println("SQLState:"+e.getSQLState());
                 System.out.println("Message :"+e.getMessage());
                 System.out.println("Vendor  :"+e.getErrorCode());
                 return -1;
               }
  }

  /**
   *
   * Get the sequence number for giving Document
   */
  public int getSeqNum(DocId did){
    if(knownDocs.get(did)!=null){
      return ((Integer)knownDocs.get(did)).intValue();
    }
    else{
      int seq = sourceSeqNum++;
      knownDocs.put(new Integer(seq),did);
      return seq;
    }
  }

  /**
   * Get sql definition for giving view name.
   */
  public String getViewDefinition(String vName){
    String qs = "SELECT definition FROM \n"+"view_definition WHERE view='"
        +vName+"';";
    try{
      java.sql.ResultSet rs = stmt.executeQuery(qs);
      while(rs.next()){
        return rs.getString("definition");
      }
      return null;
    }
    catch(SQLException e){
      System.out.println("error when get view definition");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
      return null;
    }
  }

  /**
   *
   * Get the sequence number for giving ontology.
   */
  public int getOntSeq(String ns){
    Integer in = (Integer)knownOntos.get(ns.replace('#',' ').trim());
    if(in==null)
      return -1;
    else
      return in.intValue();
  }

  /**
   * Forced exit.
   */
  public void forcedExit(){
    try{
    con.rollback();}
   catch(SQLException e){
     System.out.println("error when forced exit");
     System.out.println("SQLState:"+e.getSQLState());
     System.out.println("Message :"+e.getMessage());
     System.out.println("Vendor  :"+e.getErrorCode());
   }
  }

  /**
   * Execute query and return results.
   */
  public edu.lehigh.swat.hawk.storage.ResultSet issueQuery(edu.lehigh.swat.hawk.storage.Query qry){
    edu.lehigh.swat.hawk.storage.ResultSet rset;
    java.sql.ResultSet rs;

    Query query = (Query)qry;
    DocId persp = query.getPerspective();
    String nameSpace = getQueryOntology().concat("#");
    if(persp!=null)
      nameSpace = persp.getId().concat("#");
    int perspSeq = this.getOntSeq(nameSpace);
    //ObjectId oid = new ObjectId("");
    if(propertyTypes==null)
      getPropertyTypes();

    int i,j,ptype=0;
    Vector a = query.getAtoms();
    int l = a.size();

    StringBuffer colName = new StringBuffer("SELECT DISTINCT ");
    StringBuffer tableName = new StringBuffer("FROM ");
    StringBuffer condition = new StringBuffer("WHERE ");
    int hasname=0;
    int colnum = 0;
    Vector vcol = new Vector();
    vcol.clear();

    Vector tables = new Vector();
    tables.clear();

  //  Vector columns = new Vector();//new for alias
  //  columns.clear(); //--

      Hashtable classes = new Hashtable();
      Hashtable properties = new Hashtable();

    for(i=0;i<l;i++){
      ptype=0;
      Atom am = (Atom) a.get(i);
      String sp = am.toString();
      System.out.println(sp);

      String amp = am.getPredicate();
      String[] ama = am.getArguments();
      if(amp.startsWith("@every")||amp.startsWith("type"))
      ptype=1;
      if(amp.startsWith("@string"))
      ptype=2;

      String tn;

      switch(ptype)
      {
      case 1:
              ObjectId objid;
              if(ama[0].indexOf('#')>0||ama[0].indexOf('/')>0)
                objid = new ObjectId(ama[0]);
              else
                objid = new ObjectId(nameSpace+ama[0]);

              tn = getTableName(objid);
            //  tn = tn+"_"+perspSeq;
              tableName.append(tn).append("_view, ");
              tableName.append("url_index AS ui"+i+", ");
              colName.append("ui"+i+".url, ");
              condition.append(tn+"_view.id = ui"+i+".id AND ");
              vcol.add(ama[1]);

              classes.put(ama[1],tn+"_view.ID");

      break;
      case 2:

              //colName.append("url_index.url, url_index.id, ");
              if(properties.get(ama[0])!=null){
                String fullname = (String)properties.get(ama[0]);
                int dot = fullname.indexOf('.');
                String tname = fullname.substring(0,dot-5).toLowerCase();
                String cname = fullname.substring(dot);
                String type = "";
                try{

   /*               rs = con.getMetaData().getColumns("","",tname,cname);
                  if(rs.next())
                    type = rs.getString("TYPE_NAME");
    */
                 type = (String)propertyTypes.get(tname);
                }
                catch(Exception e)
                {
                 System.err.println(e.getMessage());
                }


                if(type.equalsIgnoreCase("object"))
                {
                  tableName.append("url_index AS ui"+i+", ");
                  condition.append("ui"+i+".url");
                  condition.append(" LIKE '").append(ama[1]).append("' AND ");
                  condition.append(properties.get(ama[0]));
                  condition.append("=").append("ui"+i+".id AND ");
                }
                else
                {
                  condition.append(properties.get(ama[0]));
                  condition.append(" LIKE '").append(ama[1]).append("' AND ");
                }
              }
      break;
      default:
              ObjectId oid;
              if(amp.indexOf('#')>0||amp.indexOf('/')>0)
                oid = new ObjectId(amp);
              else
                oid = new ObjectId(nameSpace+amp);
              tn = getTableName(oid).toLowerCase()+"_p";
          //    tn = tn+"_"+perspSeq;

              if(((String)propertyTypes.get(tn)).equals("hasinverse"))
                tn = tn+"_inv";
              if(!tables.contains(tn))
              {
                tableName.append(tn).append("_view, ");
                tables.add(tn);
              }

              if(ama[1].startsWith("?"))
              {
                if(classes.get(ama[1])!=null)
                {
                  condition.append(classes.get(ama[1])).append("=");
                  condition.append(tn).append("_view.object AND ");
                 //   else{
                 //     condition.append(select_column_name).append("=").append(ama[0]);
                 //   }
                }
                else
                {
                    String select_column_name = tn+"_view.object";
                    String type = (String)propertyTypes.get(tn);
                    if(type.equalsIgnoreCase("datatype"))
                      colName.append(select_column_name).append(", ");
                    else{
                      tableName.append("url_index AS ui"+i+", ");
                      colName.append("ui"+i+".url, ");
                      condition.append(select_column_name+" = ui"+i+".id AND ");
                    }
                    properties.put(ama[1], select_column_name);
                    vcol.add(ama[1]);
                }
              }
              else//(!ama[1].startsWith("?"))//is argument a variable?
              {
                  tableName.append("url_index AS uio"+i+", ");
                  condition.append("uio"+i+".url='").append(ama[1]).append("' AND ");
                  condition.append(tn).append("_view.object=uio"+i+".id AND ");
              }

              if(ama[0].startsWith("?"))
              {
                if(properties.get(ama[0])==null&&classes.get(ama[0]) == null)
                {
                  String select_column_name = tn + "_view.subject";
                  tableName.append("url_index AS ui" + i + ", ");
                  colName.append("ui" + i + ".url, ");
                  condition.append(select_column_name).append(" = ");
                  condition.append("ui" + i + ".id AND ");
                  vcol.add(ama[0]);
                  properties.put(ama[0], select_column_name);
                }

                if (properties.get(ama[0]) != null) {
                  condition.append(properties.get(ama[0])).append("=");
                  condition.append(tn).append("_view.subject ").append(" AND ");
                }

                if (classes.get(ama[0]) != null) {
                  condition.append(classes.get(ama[0])).append("=");
                  condition.append(tn).append("_view.subject AND ");
                }
              }
              else//(!ama[0].startsWith("?"))
              {
                  tableName.append("url_index AS uis"+i+", ");
                  condition.append("uis"+i+".url='").append(ama[0]).append("' AND ");
                  condition.append(tn).append("_view.subject=uis"+i+".id AND ");
              }

      }//end switch

   }//end for
    int len = colName.length();
    colName.delete(len-2,len);
    colName.append("\n");

    len = tableName.length();
    tableName.delete(len-2,len);
    tableName.append("\n");

    len = condition.length();
    condition.delete(len-4,len);
    condition.append("\n");

    String colNames = colName.toString();
    String tableNames = tableName.toString();
    String conditions = condition.toString();

    if(conditions.length()>6)
    System.out.println(colNames+tableNames+conditions);
    else
    System.out.println(colNames+tableNames);

    colnum = vcol.size();
    String[] varnames = new String[vcol.size()];
    for(i=0;i<colnum;i++)
    varnames[i] = (String) vcol.elementAt(i);

    rset = new edu.lehigh.swat.hawk.storage.ResultSet();


    try
    {
 //     rs.close();  //zpan
    long t0 = System.currentTimeMillis();// test
    if(conditions.length()>6)
    rs = stmt.executeQuery(colNames+tableNames+conditions);
    else
    rs = stmt.executeQuery(colNames+tableNames);
    //test
    long t1 = System.currentTimeMillis();
    long t = t1-t0;
    System.out.println("\n\ntime in SQL query: "+t+"ms.");
    //end test

    while(rs.next())
    {
      String[] data = new String[colnum];
      for(i=0;i<colnum;i++)
      {
        data[i] = rs.getString(i+1);

      }

      rset.addResult(new Result(data));
    }

//    rset.setSqlResult(rs);
  //  rs.close();
    }

     catch(SQLException e)
     {	System.out.println("error when do query");
        System.out.println(e.getMessage());
      }

   return rset;

  }

  /**
   * Get the types of properties.
   */
  public Hashtable getPropertyTypes(){
  String qs = "SELECT * \nFROM propertytypes";
  java.sql.ResultSet rs = null;
  try
  {
    rs = stmt.executeQuery(qs);
   }
  catch(SQLException e)
              {	System.out.println("error");
                      System.out.println("SQLState:"+e.getSQLState());
                      System.out.println("Message :"+e.getMessage());
                      System.out.println("Vendor  :"+e.getErrorCode());
                     // errcode = e.getErrorCode();
              }

  Hashtable propertytypes = new Hashtable();
  try{
    while (rs.next())
    {
      String pname = rs.getString("pname").toLowerCase();
      String ptype = rs.getString("type");
      propertytypes.put(pname, ptype);

    }
    rs.close();
   }catch (SQLException e)
   {
      System.out.println(e.getMessage());
   }
   propertyTypes = propertytypes;
   return propertytypes;
}

 /**
  * Decide which ontology would commit to giving the resource doc and objectId.
  */
 public int decideCommitOnto(Document doc, ObjectId id){
   String ns = id.getNameSpace().replace('#',' ').trim();
   DocId did = new DocId(ns);
   if(doc instanceof Ontology){
     Ontology on = (Ontology)doc;
     if(on.getOntClass(id)!=null||on.getProperty(id)!=null)
       return on.getOntSeq();
   }
   DocId[] dids = doc.getImports();
   if(dids==null)
     return this.getOntSeq(ns);
   int len = dids.length;
   for(int i=0;i<len;i++){
     if(dids[i].equals(did)){
       return this.getOntSeq(ns);
     }
     edu.lehigh.swat.hawk.core.Ontology ont = (edu.lehigh.swat.hawk.core.Ontology)this.getDocument(dids[i].getId());
     if(ont.getOntClass(id)!=null||ont.getProperty(id)!=null)
       return this.getOntSeq(dids[i].getId());
   }
   return this.getOntSeq(ns);
 }

 /**
  * Set the query ontology.
  */
  public void setQueryOntology(String uri){
    queryOnto = uri;
  }

  /**
   * Get the query ontology.
   */
  public String getQueryOntology(){
    return queryOnto;
  }

  /**
   * Get the database name.
   */
  public String getDBName(){
    return dbName;
  }

  protected int ontSeqNum=0, sourceSeqNum=0, maxUrlNum=0;
  protected Hashtable knownOntos, knownDocs, knownUrls, propertyTypes;
  private String dbName, queryOnto;
  private Connection con;
  protected Statement stmt;
  private Hashtable cachedDocs, loadedDocs;
  private boolean serializeOnto = true;
  private String emptyTableState = "";
}
