/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/


package edu.lehigh.swat.hawk.storage.simpledb;

import edu.lehigh.swat.hawk.core.DocId;
import edu.lehigh.swat.hawk.core.ObjectId;
import java.sql.*;
import java.util.*;

/**
 * <p>encapsulating the web ontology </p>
 * @author Z. Pan
 * @version 1.0
 */

public class Ontology extends Document implements edu.lehigh.swat.hawk.core.Ontology{

  /**
   * Constructor
   */
  public Ontology(edu.lehigh.swat.hawk.storage.simplememory.Ontology mOnto) {
    memoryOnto = mOnto;
  }

  /**
   * Get the memory-based ontology object.
   */
  public edu.lehigh.swat.hawk.storage.simplememory.Ontology getMemOnto(){
    return memoryOnto;
  }

  /**
   * get the ID of this document
   */
  public DocId getID(){
    return memoryOnto.getID();
  }

  /**
   * set the ID of this document
   */
  public void setID(DocId id){
    memoryOnto.setID(id);
  }

  /**
   * Add an imports statement.
   */
  public void addImports(DocId id){
    super.addImports(id);
    memoryOnto.addImports(id);
    Document idoc = (Document)model.getDocument(id.getId());
    if(!(idoc instanceof Ontology))
      return;
    Ontology ionto = (Ontology)idoc;
    edu.lehigh.swat.hawk.core.OntClass[] ics = ionto.getAllPlusImportedClasses();
    int len = ics.length;
    for(int i=0;i<len;i++){
      edu.lehigh.swat.hawk.core.OntClass icls = ics[i];
      edu.lehigh.swat.hawk.core.OntClass oc = createClass();
      oc.setID(icls.getID());
      addImportedClass(icls.getID(),oc);
    }
    edu.lehigh.swat.hawk.core.OntProperty[] ips = ionto.getAllPlusImportedProperties();
    int plen = ips.length;
    for(int i=0;i<plen;i++){
      edu.lehigh.swat.hawk.core.OntProperty ip = ips[i];
      edu.lehigh.swat.hawk.core.OntProperty op = createProperty();
      op.setID(ip.getID());
      addImportedProperty(ip.getID(),op);
    }
  }


  /**
   * add a class to this ontology
   */
  public void addOntClass(edu.lehigh.swat.hawk.core.OntClass c){
    memoryOnto.addOntClass(c);
    String tableName = model.getTableName(c.getID());
    if(tableName==null)
      return;
    String qs = "CREATE TABLE "+tableName
    +" (\nID INTEGER,\nSOURCE INTEGER,ONTO INTEGER,\nPRIMARY KEY(ID,SOURCE)\n)";

    try{
      stmt.executeUpdate(qs);
    }
     catch(SQLException e){
                System.out.println("error when store class");
                System.out.println(qs);
                System.out.println("SQLState:"+e.getSQLState());
                System.out.println("Message :"+e.getMessage());
                System.out.println("Vendor  :"+e.getErrorCode());

              }

      String qs2 = "";
      String viewName = tableName+"_view";

      qs2 = "CREATE VIEW "+viewName+" AS\n SELECT * FROM "+tableName;


      String qs3 = "";
      try{
        qs3 = "INSERT INTO view_definition VALUES ('"+viewName+"', '"+qs2+"')";
        stmt.executeUpdate(qs2);
        stmt.executeUpdate(qs3);
      }
      catch(SQLException e){
        System.out.println("error when store class, create view");
        System.out.println(qs2);
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
      }
  }


  /**
   * get the specified class
   */
  public edu.lehigh.swat.hawk.core.OntClass getOntClass(ObjectId id){
    edu.lehigh.swat.hawk.core.OntClass oc = memoryOnto.getOntClass(id);
    return oc;
  }

  /**
   * delete the specified class
   */
  public void deleteOntClass(ObjectId id){
    memoryOnto.deleteOntClass(id);
  }

  /**
   * get all the classes in this ontology
   */
  public edu.lehigh.swat.hawk.core.OntClass[] getAllClasses(){
    return memoryOnto.getAllClasses();

  }

  /**
   * Get All the classes plus the imported Classes.
   */
  public edu.lehigh.swat.hawk.core.OntClass[] getAllPlusImportedClasses(){
    return memoryOnto.getAllPlusImportedClasses();
  }


  /**
   * Get the Iterator of all the classes.
   */
  public Iterator getClassIterator(){
    return memoryOnto.getClassIterator();
  }


  /**
   * add a property to this ontology
   */
  public void addProperty(edu.lehigh.swat.hawk.core.OntProperty prop){
    memoryOnto.addProperty(prop);

    String tableName = model.getTableName(prop.getID());
    if(tableName==null)
      return;
    String qs = "";
    String pinfo = "INSERT INTO propertytypes VALUES('";
    tableName = tableName+"_p";
    if((prop.isDatatype()!=null && !prop.isDatatype().booleanValue())||
        prop.isFunctional()||prop.isTransitive()||prop.isInverseFunctional()||
         prop.isSymmetric()){
      qs = "CREATE TABLE "+tableName
    +" (\nSUBJECT INTEGER,\nOBJECT INTEGER,\nSOURCE INTEGER,\nONTO INTEGER, \nPRIMARY KEY(SUBJECT,OBJECT,SOURCE)\n)";
      pinfo += tableName+"', 'object')";
    }
    else{
      qs = "CREATE TABLE "+tableName
    +" (\nSUBJECT INTEGER,\nOBJECT VARCHAR (200),\nSOURCE INTEGER,\nONTO INTEGER,\nPRIMARY KEY(SUBJECT,OBJECT,SOURCE)\n)";
      pinfo += tableName+"', 'datatype')";
    }
   // log.println(qs);

    try{
      stmt.executeUpdate(qs);
      stmt.executeUpdate(pinfo);
    }
    catch(SQLException e){
                //log.println("error when store property");
      System.out.println("error when store property");
      System.out.println("SQLState:"+e.getSQLState());
      System.out.println("Message :"+e.getMessage());
      System.out.println("Vendor  :"+e.getErrorCode());
    }

      String qs2 = "";
      String viewName = tableName+"_view";
  //    if(viewSeqNum == currentSeqNum){

        qs2 = "CREATE VIEW "+viewName+" AS\n SELECT * FROM "+tableName;


      String qs3 = "";
      try{
        qs3 = "INSERT INTO view_definition VALUES ('"+viewName+"', '"+qs2+"')";
        stmt.executeUpdate(qs2);
        stmt.executeUpdate(qs3);
      }
      catch(SQLException e){
              //  log.println("error when store class, create view");
        System.out.println(qs2);
        System.out.println("error when store class, create view");
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
      }
  }

  /**
   * get the specified property
   */
  public edu.lehigh.swat.hawk.core.OntProperty getProperty(ObjectId id){
    return memoryOnto.getProperty(id);
  }

  /**
   * delete a specified property
   */
  public void deleteProperty(ObjectId id){
    memoryOnto.deleteProperty(id);
  }

  /**
   * get all the properties in this ontology
   */
  public edu.lehigh.swat.hawk.core.OntProperty[] getAllProperties(){
    return memoryOnto.getAllProperties();
  }

  /**
   * Get All the classes plus the imported properties.
   */
  public edu.lehigh.swat.hawk.core.OntProperty[] getAllPlusImportedProperties(){
    return memoryOnto.getAllPlusImportedProperties();
  }

  /**
   * Get the Iterator of all the properties.
   */
  public Iterator getPropertyIterator(){
    return memoryOnto.getPropertyIterator();
  }


  /**
   * add a property to this ontology
   */
  public void addRestriction(edu.lehigh.swat.hawk.core.PropertyRestriction p){
    memoryOnto.addRestriction(p);
  }

  /**
   * get the specified property
   */
  public edu.lehigh.swat.hawk.core.PropertyRestriction getRestriction(ObjectId id){
    return memoryOnto.getRestriction(id);
  }

  /**
   * delete a specified property
   */
  public void deleteRestriction(ObjectId id){
    memoryOnto.deleteRestriction(id);
  }

  /**
   * get all the properties in this ontology
   */
  public edu.lehigh.swat.hawk.core.PropertyRestriction[] getAllRestrictions(){
    return memoryOnto.getAllRestrictions();
  }

  /**
   * Get the priorVersion ontologies which are backwardCompatibleWith
   */
  public DocId[] getCompatiblePriorOnts(){
    DocId[] dIds = memoryOnto.getCompatiblePriorOnts();
    return dIds;
  }

  /**
   * Add a priorVersion ontology which is backwardCompatibleWith.
   */
  public void addCompatiblePriorOnts(DocId id){
    memoryOnto.addCompatiblePriorOnts(id);
  }

  /**
   * Delete a priorVersion ontology which is backwardCompatibleWith.
   */
  public void deleteCompatiblePriorOnts(DocId id){
    memoryOnto.deleteCompatiblePriorOnts(id);
  }

  /**
   * Get the priorVersion ontologies may be incompatibleWith
   */
  public DocId[] getPriorOnts(){
    return memoryOnto.getPriorOnts();
  }

  /**
   * Add a priorVersion ontology may be incompatibleWith.
   */
  public void addPriorOnts(DocId id){
    memoryOnto.addPriorOnts(id);
  }

  /**
   * Delete a priorVersion ontology may be incompatibleWith.
   */
  public void deletePriorOnts(DocId id){
    memoryOnto.deletePriorOnts(id);
  }

  /**
   * get the version of this ontology
   */
  public String getVersion(){
    return memoryOnto.getVersion();
  }

  /**
   * Set the version of this ontology.
   */
  public void setVersion(String ver){
    memoryOnto.setVersion(ver);
  }


  /**
   * Create a Class object.
   */
  public edu.lehigh.swat.hawk.core.OntClass createClass(){
    return memoryOnto.createClass();
  }

  /**
   * Create a Property object.
   */
  public edu.lehigh.swat.hawk.core.OntProperty createProperty(){
    return memoryOnto.createProperty();
  }

  /**
   * Create a PropertyRestriction object.
   */
  public edu.lehigh.swat.hawk.core.PropertyRestriction createRestriction(){
    return memoryOnto.createRestriction();
  }

  /**
   * Flush the axioms into database.
   */
  public void flush(){
    super.flush();
    dbName = ((DatabaseStorage)getModel()).getDBName();
    flushSubs();
    flushEquivals();
    flushInverseOf();
  }

  private void flushSubs(){
    //deal with subClass  //pzx: changed to all plus imports on 10/12/2004
    edu.lehigh.swat.hawk.core.OntClass[] classes = this.getAllPlusImportedClasses();
    int cNum = classes.length;

    for(int i=0;i<cNum;i++){
      edu.lehigh.swat.hawk.core.OntClass cls = classes[i];
      if(cls.getID().getLocalName()==null||cls.getID().getLocalName()=="")
        continue;
      ObjectId[] sid = cls.getDirectSubClasses();
      String className = model.getTableName(cls.getID());
  //    int oriSeq = model.getOntSeq(cls.getID().getNameSpace());
      int sn =0;
      if(sid!=null)
        sn = sid.length;
      if(sn<=0)
        continue;
      String vName = className+"_view";
      String dv = "DROP TABLE "+vName;
      StringBuffer cvb;

      String def = model.getViewDefinition(vName);
      cvb = new StringBuffer(def);

      for(int j=0;j<sn;j++){
   //     int sOriSeq = model.getOntSeq(sid[j].getNameSpace());
   //     if(sOriSeq!=oriSeq)
   //       continue;
        String tName = model.getTableName(sid[j]);
        if(tName==null)
          continue;
        cvb.append(" UNION SELECT * FROM ");
        String avName = tName+"_view";
        String vdef = model.getViewDefinition(avName);
        if(vdef!=null){
          int ind = vdef.indexOf(" FROM");
          vdef = vdef.substring(ind+5);
        }
        if(vdef!=null&&vdef.indexOf(" "+className)>0){
          int i1 = vdef.indexOf(" "+className);
          int i2 = vdef.indexOf(' ',i1+1);
          if(i2<=0)
            i2 = vdef.length();
          int i0 = i1-20;
          StringBuffer sb = new StringBuffer(vdef);
          sb.delete(i0,i2);
          cvb.append(sb);
        }
        else
          cvb.append(tName).append("_view");
      }
      try{
        String cv = cvb.toString();
        if(dbName.endsWith(".mdb")){
          stmt.execute(dv);
          stmt.execute(cv);
        }else if(dbName.indexOf("postgresql")>0)
          stmt.execute(cvb.insert(7,"OR REPLACE ").toString());
        String qs3 = "UPDATE view_definition SET definition='"+cv+"'"+
             "WHERE view='"+vName+"';";
        stmt.execute(qs3);

      }
      catch(SQLException e){
        System.out.println("error when recreate class hierarchy");
        System.out.println(cvb.toString());
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
      }
    }

   // deal with subProperty
   edu.lehigh.swat.hawk.core.OntProperty[] Properties
       = this.getAllPlusImportedProperties();
   int pNum = Properties.length;

   for(int i=0;i<pNum;i++){
     edu.lehigh.swat.hawk.core.OntProperty cls = Properties[i];
     if(cls.getID().getLocalName()==null||cls.getID().getLocalName()=="")
       continue;
     ObjectId[] sid = cls.getDirectSubProperties();
     String propertyName = model.getTableName(cls.getID())+"_p";
   //  int oriSeq = model.getOntSeq(cls.getID().getNameSpace());
     if(sid==null)
       continue;
     int sn = sid.length;
     if(sn<=0)
       continue;
     String vName = propertyName+"_view";
     String dv = "DROP TABLE "+vName;
     StringBuffer cvb;

     String def = model.getViewDefinition(vName);
     cvb = new StringBuffer(def);

     for(int j=0;j<sn;j++){
   //    int sOriSeq = model.getOntSeq(sid[j].getNameSpace());
   //   if(sOriSeq!=oriSeq)
  //        continue;
       String tName = model.getTableName(sid[j]);
       if(tName==null)
         continue;
       tName = tName + "_p";
       cvb.append(" UNION SELECT * FROM ");
       String avName = tName+"_view";
       String vdef = model.getViewDefinition(avName);
       if(vdef!=null){
         int ind = vdef.indexOf(" FROM");
         vdef = vdef.substring(ind+5);
       }
       if(vdef!=null&&vdef.indexOf(" "+propertyName)>0){
         int i1 = vdef.indexOf(" "+propertyName);
         int i2 = vdef.indexOf(' ',i1+1);
         if(i2<=0)
           i2 = vdef.length();
         int i0 = i1-20;
         StringBuffer sb = new StringBuffer(vdef);
         sb.delete(i0,i2);
         cvb.append(sb);
       }
       else
         cvb.append(tName).append("_view");
     }
     try{
       String cv = cvb.toString();
       if(dbName.endsWith(".mdb")){
         stmt.execute(dv);
         stmt.execute(cv);
       }else if(dbName.indexOf("postgresql")>0)
         stmt.execute(cvb.insert(7,"OR REPLACE ").toString());

       String qs3 = "UPDATE view_definition SET definition='"+cv+"'"+
            "WHERE view='"+vName+"';";
       stmt.execute(qs3);

     }
     catch(SQLException e){
       System.out.println("error when recreate property hierarchy");
       System.out.println(cvb.toString());
       System.out.println("SQLState:"+e.getSQLState());
       System.out.println("Message :"+e.getMessage());
       System.out.println("Vendor  :"+e.getErrorCode());
     }
   }
  }

  private void flushEquivals(){
    //deal with equivalentClass
    edu.lehigh.swat.hawk.core.OntClass[] classes = this.getAllClasses();
    int cNum = classes.length;
    for(int i=0;i<cNum;i++){
      edu.lehigh.swat.hawk.core.OntClass cls = classes[i];
      if(cls.getID().getLocalName()==null||cls.getID().getLocalName()=="")
        continue;
      ObjectId[] eid = cls.getEquivalentClasses();
      String className = model.getTableName(cls.getID());
    //  int oriSeq = model.getOntSeq(cls.getID().getNameSpace());
      if(eid==null)
        continue;
      int sn = eid.length;
      if(sn<=0)
        continue;
      String vName = className+"_view";
      String dv = "DROP TABLE "+vName;
      StringBuffer cvb;
      String start = model.getViewDefinition(vName);
      cvb = new StringBuffer(start);

      for(int j=0;j<sn;j++){
     //   int sOriSeq = model.getOntSeq(eid[j].getNameSpace());
      //  if(sOriSeq!=oriSeq)
      //    continue;

        String tName = model.getTableName(eid[j]);
        if(tName == null)
          continue;
        cvb.append(" UNION SELECT * FROM ");
        String avName = tName+"_view";
        String vdef = model.getViewDefinition(avName);
        if(vdef!=null){
          int ind = vdef.indexOf(" FROM");
          vdef = vdef.substring(ind+5);
        }
        cvb.append(vdef);
      }
      try{
        String cv = cvb.toString();
        if(dbName.endsWith(".mdb")){
         stmt.execute(dv);
         stmt.execute(cv);
       }else if(dbName.indexOf("postgresql")>0)
         stmt.execute(cvb.insert(7,"OR REPLACE ").toString());

        String qs3 = "UPDATE view_definition SET definition='"+cv+"'"+
            "WHERE view='"+vName+"';";
        stmt.execute(qs3);

      }
      catch(SQLException e){
        System.out.println("error when recreate class hierarchy");
        System.out.println(cvb.toString());
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
      }
    }

    //deal with equivalentProperty
    edu.lehigh.swat.hawk.core.OntProperty[] Properties = this.getAllProperties();
    int pNum = Properties.length;
    for(int i=0;i<pNum;i++){
      edu.lehigh.swat.hawk.core.OntProperty cls = Properties[i];
      if(cls.getID().getLocalName()==null||cls.getID().getLocalName()=="")
        continue;
      ObjectId[] eid = cls.getEquivalentProperties();
      String PropertyName = model.getTableName(cls.getID())+"_p";
  //    int oriSeq = model.getOntSeq(cls.getID().getNameSpace());
      if(eid==null)
        continue;
      int sn = eid.length;
      if(sn<=0)
        continue;
      String vName = PropertyName+"_view";
      String dv = "DROP TABLE "+vName;
      StringBuffer cvb;
      String start = model.getViewDefinition(vName);
      cvb = new StringBuffer(start);

      for(int j=0;j<sn;j++){
    //    int sOriSeq = model.getOntSeq(eid[j].getNameSpace());
     //   if(sOriSeq!=oriSeq)
     //     continue;

        String tName = model.getTableName(eid[j]);
        if(tName == null)
          continue;
        tName = tName + "_p";
        cvb.append(" UNION SELECT * FROM ");
        String avName = tName+"_view";
        String vdef = model.getViewDefinition(avName);
        if(vdef!=null){
          int ind = vdef.indexOf(" FROM");
          vdef = vdef.substring(ind+5);
        }
        cvb.append(vdef);
      }
      try{
        String cv = cvb.toString();
        if(dbName.endsWith(".mdb")){
         stmt.execute(dv);
         stmt.execute(cv);
       }else if(dbName.indexOf("postgresql")>0)
         stmt.execute(cvb.insert(7,"OR REPLACE ").toString());

        String qs3 = "UPDATE view_definition SET definition='"+cv+"'"+
            "WHERE view='"+vName+"';";
        stmt.execute(qs3);

      }
      catch(SQLException e){
        System.out.println("error when recreate Property hierarchy");
        System.out.println(cvb.toString());
        System.out.println("SQLState:"+e.getSQLState());
        System.out.println("Message :"+e.getMessage());
        System.out.println("Vendor  :"+e.getErrorCode());
      }
    }
  }

  /**
   * Flush owl:inverseOf into db view.
   */
  private void flushInverseOf(){
    edu.lehigh.swat.hawk.core.OntProperty[] Properties
        = this.getAllPlusImportedProperties();
    int pNum = Properties.length;

    for (int i = 0; i < pNum; i++) {
      edu.lehigh.swat.hawk.core.OntProperty cls = Properties[i];
      if (cls.getID().getLocalName() == null || cls.getID().getLocalName() == "")
        continue;
      ObjectId[] sid = cls.getInverseOf();
      String propertyName = model.getTableName(cls.getID())+"_p";
  //  int oriSeq = model.getOntSeq(cls.getID().getNameSpace());
      if (sid == null)
        continue;
      int sn = sid.length;
      if (sn <= 0)
        continue;
      String vName = propertyName  + "_inv"+ "_view";
      String dv = "DROP TABLE " + vName;
      StringBuffer cvb;

      String def = model.getViewDefinition(vName);
      if(def==null)
        cvb = new StringBuffer("CREATE VIEW " + vName + " AS SELECT * FROM "
                               + propertyName + "_view ");
      else
        cvb = new StringBuffer(def);

      for (int j = 0; j < sn; j++) {
  //    int sOriSeq = model.getOntSeq(sid[j].getNameSpace());
  //   if(sOriSeq!=oriSeq)
  //        continue;
        String tName = model.getTableName(sid[j]);
        if (tName == null)
          continue;
        tName = tName + "_p_view";
        cvb.append(" UNION SELECT ");
        cvb.append(tName + ".OBJECT AS SUBJECT, ").append(tName
              + ".SUBJECT AS OBJECT, "+tName+".source, "+tName
              +".onto "+"FROM ").append(tName);
      }
      try {
        String cv = cvb.toString();
        if (dbName.endsWith(".mdb")) {
          if(def!=null)
            stmt.execute(dv);
          stmt.execute(cv);
        }
        else if (dbName.indexOf("postgresql") > 0)
          stmt.execute(cvb.insert(7, "OR REPLACE ").toString());

        String qs3 = "UPDATE view_definition SET definition='" + cv + "'" +
            "WHERE view='" + vName + "';";
        stmt.execute(qs3);
        String qs4 = "UPDATE propertytypes SET type='hasinverse' WHERE pname='"
            +propertyName+"';";
        stmt.execute(qs4);
      }
      catch (SQLException e) {
        System.out.println("error when recreate property hierarchy");
        System.out.println(cvb.toString());
        System.out.println("SQLState:" + e.getSQLState());
        System.out.println("Message :" + e.getMessage());
        System.out.println("Vendor  :" + e.getErrorCode());
      }
    }
  }

  /**
   * Add an imported class.
   */
  public void addImportedClass(ObjectId cid, edu.lehigh.swat.hawk.core.OntClass c){
    memoryOnto.addImportedClass(cid,c);
  }

  /**
   * Add an imported property.
   */
  public void addImportedProperty(ObjectId pid, edu.lehigh.swat.hawk.core.OntProperty p){
    memoryOnto.addImportedProperty(pid,p);
  }


  /**
   * To see if ontology has the class.
   */
  public boolean hasClass(String name){
    return memoryOnto.hasClass(name);
  }

  /**
   * Get the imported property.
   */
  public boolean hasProperty(String name){
    return memoryOnto.hasProperty(name);
  }

  /**
   * Get the table names in the view definition
   */
  protected Vector viewDefToTabNames(String vdef){
    StringTokenizer st = new StringTokenizer(vdef);
    Vector v = new Vector();
    String t;
    while(st.hasMoreTokens()){
      t = st.nextToken();
      if(t.equalsIgnoreCase("from")){
        String viewName = st.nextToken();
        int ind1 = viewName.indexOf('_');
        int ind2 = viewName.lastIndexOf('_');
        if(ind1!=ind2){
          ind2 = viewName.indexOf('_',ind1+1);
          viewName = viewName.substring(0,ind2);
        }
        v.add(viewName);
      }
    }
    return v;
  }

  /**
   * Set the document sequence number.
   */
  public void setOntSeq(int i){
    ontSeq = i;
  }

  /**
   * Get the document sequence number.
   */
  public int getOntSeq(){
    return ontSeq;
  }

  /**
   * To tell if the ontology imorted specified class.
   */
  public boolean importedClass(ObjectId cid){
    return memoryOnto.importedClass(cid);
  }

  /**
   * To tell if the ontology imported specified property.
   */
  public boolean importedProperty(ObjectId pid){
    return memoryOnto.importedProperty(pid);
  }

  /**
   * Get the imported class.
   */
//  public edu.lehigh.swat.hawk.core.OntClass getImportedClass(ObjectId id){
//    return memoryOnto.getImportedClass(id);
//  }

  /**
   * To see if ontology has imported the property.
   */
//  public edu.lehigh.swat.hawk.core.OntProperty getImportedProperty(ObjectId id){
//    return memoryOnto.getImportedProperty(id);
//  }

  protected int ontSeq;
  protected edu.lehigh.swat.hawk.storage.simplememory.Ontology memoryOnto;
//  private Vector compatibleOntos;
  private String dbName;
}
