/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.simpledb;

/**
 * The interface of kif-like format query.
 * @author Z. Pan
 * @version 1.0
 */
import java.util.Vector;
import edu.lehigh.swat.hawk.core.DocId;
import java.io.*;

public class Query extends edu.lehigh.swat.hawk.storage.Query{
  Vector atoms;
  DocId perspective;
  /**
   * Constructor
   */
  public Query() {
    atoms = new Vector();
  }

  /** Adds an additional atom as a conjunct to the existing sentence
  where the input is the atom to be added. */
  public void addConjunct(Atom atom) {
    atoms.add(atom);
  }

  /** Returns the vector of atoms that are conjuncted together
  in the sentence. */
  public Vector getAtoms() {
    return atoms;
  }

  /**
   * Set the perspective ontology's id.
   */
  public void setPerspective(DocId pid){
    perspective = pid;
  }

  /**
   * Get the id of the perspective ontology.
   */
  public DocId getPerspective(){
    return perspective;
  }

  /**
   * Parse a string to query.
   * @param s
   * @return
   */
  public static Query StringToQuery(String s){
    try{
      LineNumberReader lr = new LineNumberReader(new StringReader(s));
      Query query = new Query();
      String line;
      while((line=lr.readLine())!=null){
        if(line.startsWith("[")){
          line = line.substring(1,line.length()-1);
          DocId did = new DocId(line);
          query.setPerspective(did);
          continue;
        }
        int end = line.lastIndexOf(')');
        line = line.substring(1,end);
        String[] args = line.split(" ");
        if(args.length!=3){
          System.out.println("syntax error at "+lr.getLineNumber());
          return null;
        }
        Atom atom = new Atom(args[0],args[1],args[2]);
        query.addConjunct(atom);
      }
      return query;
    }
    catch(Exception e){
      e.printStackTrace();
    }
    return null;
  }

  public String toString(){
    String s="";
    int len = atoms.size();
    for(int i=0;i<len;i++){
      Atom a = (Atom)atoms.get(i);
      if(a!=null)
      s = s+a.toString()+"\n";
    }
    return s;
  }
}
