/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.simplememory;
import edu.lehigh.swat.hawk.core.DocId;
import edu.lehigh.swat.hawk.core.ObjectId;
import java.util.*;

/**
 * A simple implementation of core.Document in memory.
 * @author Z. Pan
 * @version 1.0
 */

public class Document extends SerializableClass implements edu.lehigh.swat.hawk.core.Document{

  /**
   * Constructor
   */

  public Document() {
    assertions = new Vector();
    imports = new Vector();
    distinctSets = new Vector();
    sameSets = new Vector();
    individuals = new Hashtable();
    comments = new Vector();
    nameSpaces = new HashMap();
  }

  /**
   * Add an assertion to this document.
   * (use individuals' properties as back-end)
   */
  public void addAssertion(ObjectId subject, ObjectId predicate, ObjectId object){
    edu.lehigh.swat.hawk.core.Individual indiv = this.getIndividual(subject);
    if(indiv!=null)
      indiv.addProperty(predicate,object);
  }

  /**
   * Delete an assertion in this document.
   * (use individuals' properties as back-end)
   */
  public void deleteAssertion(edu.lehigh.swat.hawk.core.Assertion a){
    ObjectId subject = a.getSubject();
    ObjectId predicate = a.getPredicate();
    ObjectId object = a.getObject();
    edu.lehigh.swat.hawk.core.Individual indiv = this.getIndividual(subject);
    if(indiv!=null)
      indiv.deleteProperty(predicate,object);
  }

  /**
   * Get all assertions.
   * (use individuals' properties as back-end)
   */
  public edu.lehigh.swat.hawk.core.Assertion[] getAllAssertions(){
    Vector assertions = new Vector();
    edu.lehigh.swat.hawk.core.Individual[] indivs = this.getAllIndividuals();
    int len = indivs.length;
    for(int i=0;i<len;i++){
      edu.lehigh.swat.hawk.core.Individual indiv = indivs[i];
      ObjectId iid = indiv.getID();
      ObjectId[] pids = indiv.getProperties();
      int plen = pids.length;
      for(int j=0;j<plen;j++){
        ObjectId pid = pids[j];
        ObjectId[] vids = indiv.getValuesForProperty(pid);
        int vlen = vids.length;
        for(int k=0;k<vlen;k++){
          ObjectId obj = vids[k];
          Assertion a = new Assertion(iid,pid,obj);
          assertions.add(a);
        }
      }
    }
    int n = assertions.size();
    Assertion[] aa = new Assertion[n];
    Object[] oa = assertions.toArray();
    System.arraycopy(oa,0,aa,0,n);
    return aa;
  }

  /**
   * Get the Iterator of all the assertions.
   */
  public Iterator getAssertionIterator(){
    Vector assertions = new Vector();
    edu.lehigh.swat.hawk.core.Individual[] indivs = this.getAllIndividuals();
    int len = indivs.length;
    for(int i=0;i<len;i++){
      edu.lehigh.swat.hawk.core.Individual indiv = indivs[i];
      ObjectId iid = indiv.getID();
      ObjectId[] pids = indiv.getProperties();
      int plen = pids.length;
      for(int j=0;j<plen;j++){
        ObjectId pid = pids[j];
        ObjectId[] vids = indiv.getValuesForProperty(pid);
        int vlen = vids.length;
        for(int k=0;k<vlen;k++){
          ObjectId obj = vids[k];
          Assertion a = new Assertion(iid,pid,obj);
          assertions.add(a);
        }
      }
    }
    return assertions.iterator();
  }


  /**
   * Get the imports statements.
   */
  public DocId[] getImports(){
    if(imports.size()<=0)
      return null;
    else{
     Object[] oa = imports.toArray();
     int len = oa.length;
     DocId[] oc = new DocId[len];
     System.arraycopy(oa,0,oc,0,len);
     return oc;
    }

  }

  /**
   * Add an imports statement.
   */
  public void addImports(DocId id){
    imports.add(id);
  }

  /**
   * Delete an imports statement.
   */
  public void deleteImports(DocId id){
    imports.removeElement(id);
  }

  /**
   * get the ID of this document
   */
  public DocId getID(){
    return docId;
  }

  /**
   * set the ID of this document
   */
  public void setID(DocId id){
    docId = id;
  }

  /**
   * Add a set of distinct individuals' IDs
   */
  public void addDistinctIndividualsSet(Collection dis){
    distinctSets.add(dis);
  }

  /**
   * delete a set of distinct individuals' IDs
   */
  public void deleteDistinctIndividualsSet(Collection dis){
    distinctSets.remove(dis);
  }

  /**
   * get sets of distinct individuals' IDs
   */
  public Collection[] getDistinctIndividualsSets(){
    Object[] oa = distinctSets.toArray();
    int len = oa.length;
    Collection[] oc = new Collection[len];
    System.arraycopy(oa,0,oc,0,len);
    return oc;
  }

  /**
   * Add a set of equivalent individuals' IDs
   */
  public void addSameIndividualsSet(Collection dis){
    sameSets.add(dis);
  }

  /**
   * delete a set of equivalent individuals' IDs
   */
  public void deleteSameIndividualsSet(Collection dis){
    sameSets.remove(dis);
  }

  /**
   * get sets of equivalent individuals' IDs
   */
  public Collection[] getSameIndividualsSets(){
    Object[] oa = sameSets.toArray();
    int len = oa.length;
    Collection[] oc = new Collection[len];
    System.arraycopy(oa,0,oc,0,len);
    return oc;
  }

  /**
   * Create a Individual object.
   */
  public edu.lehigh.swat.hawk.core.Individual createIndividual(){
    return new Individual();
  }


  /**
   * Add an individual.
   */
  public void addIndividual(edu.lehigh.swat.hawk.core.Individual indi){
    individuals.put(indi.getID(),indi);
  }

  /**
   * Delete an individual.
   */
  public void deleteIndividual(edu.lehigh.swat.hawk.core.Individual indi){
    individuals.remove(indi.getID());
  }

  /**
   * Get an individual.
   */
  public edu.lehigh.swat.hawk.core.Individual getIndividual(ObjectId id){
    if(individuals.containsKey(id))
      return (edu.lehigh.swat.hawk.core.Individual)individuals.get(id);
    else
      return null;
  }

  /**
   * Get all the individuals.
   */
  public edu.lehigh.swat.hawk.core.Individual[] getAllIndividuals(){
    Collection c = individuals.values();
    Object[] oa = c.toArray();
    int len = oa.length;
    edu.lehigh.swat.hawk.core.Individual[] oc = new edu.lehigh.swat.hawk.core.Individual[len];
    System.arraycopy(oa,0,oc,0,len);
    return oc;
  }

  /**
   * Get the Iterator of all the individuals.
   */
  public Iterator getIndividualIterator(){
    return individuals.values().iterator();
  }


  /**
   * Set the nameSpaces of this document.
   */
  public void setNameSpaces(HashMap ns){
    nameSpaces = ns;
  }

  /**
   * Get the nameSpaces of this document.
   */
  public HashMap getNameSpaces(){
    return nameSpaces;
  }

  /**
   * Set the xml:Base of this document.
   */
  public void setXMLBase(String base){
    xmlBase = base;
  }

  /**
   * Get the xml:Base of this document.
   */
  public String getXMLBase(){
    return xmlBase;
  }

  /**
   * Add a description.
   */
  public void addDescription(String s){
    comments.add(s);
  }

  /**
   * Delete a description.
   */
  public void deleteDescriprion(String s){
    comments.remove(s);
  }


  /**
   * Get the first description.
   */
  public String getDescription(){
    if(comments.size()==0)
      return null;
    else
      return (String)comments.elementAt(0);
  }

  /**
   * Get all the descriptions.
   */
  public Iterator getAllDescriptions(){
    return comments.iterator();
  }

  /**
   * Set the label.
   */
  public void setLabel(String s){
    label = s;
  }

  /**
   * Get the label.
   */
  public String getLabel(){
    return label;
  }

  /**
   * Set descendants.
   */
  public void setDescendants(Stack des){

  }

  /**
   * Get descendants.
   */
  public Stack getDescendants(){
    return null;
  }

  /**
   * Flush the document.
   * Do nothing for memory model.
   */
  public void flush(){

  }

  private Vector assertions, imports, distinctSets, sameSets, comments;
  private Hashtable individuals;
  private DocId docId;
  private HashMap nameSpaces;
  private String xmlBase, label;

}
