/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.simplememory;
import java.util.*;
import edu.lehigh.swat.hawk.core.ObjectId;
/**
 * A simple implementation of core.Individual in memory.
 * @author Z. Pan
 * @version 1.0
 */

public class Individual extends OntObject implements edu.lehigh.swat.hawk.core.Individual{

  /**
   * Constructor
   */
  public Individual() {
    properties = new Hashtable();
    source = new Vector();
    sameSet = null;
    sameAs = new Vector();
    differentFrom = new Vector();
  }

  /**
   * Get the sourece classes.
   */
  public ObjectId[] getSourceClasses(){
    int len = source.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = source.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add a source class.
   */
  public void addSourceClass(ObjectId cid){
    source.add(cid);
  }

  /**
   * Delete a source class.
   */
  public void deleteSourceClass(Object cid){
    source.remove(cid);
  }

  /**
   * Add a related property.
   */
  public void addProperty(ObjectId pid, ObjectId v){
    if(properties.containsKey(pid)){
      Vector vec = (Vector)properties.get(pid);
      vec.add(v);
    }
    else{
      Vector vec = new Vector();
      vec.add(v);
      properties.put(pid,vec);
    }
  }

  /**
   * Delete a related property.
   */
  public void deleteProperty(ObjectId pid, ObjectId v){
    if(properties.containsKey(pid)){
      Vector vec = (Vector)properties.get(pid);
      vec.remove(v);
    }
  }

  /**
   * Get all the related properties.
   */
  public ObjectId[] getProperties(){
    int len = properties.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = properties.keySet().toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }


  /**
   * Get all the values on specfied property.
   */
  public edu.lehigh.swat.hawk.core.ObjectId[] getValuesForProperty(ObjectId pid){
    if(properties.containsKey(pid)){
      Vector vec = (Vector)properties.get(pid);
      Object[] oa = vec.toArray();
      int len = vec.size();
      ObjectId[] od = new ObjectId[len];
      System.arraycopy(oa,0,od,0,len);
      return od;
    }
    else
      return null;
  }

  /**
   * Add a id which is same as this individual.
   */
  public void addSameAs(ObjectId id){
    sameAs.add(id);
  }

  /**
   * Delete a id which is same as this individual.
   */
  public void deleteSameAs(ObjectId id){
    sameAs.remove(id);
  }

  /**
   * Get all the ids that are same as this individual.
   */
  public ObjectId[] getAllSameAs(){
    if(sameAs.size()<=0)
      return null;
    Object[] oa = sameAs.toArray();
    int len = oa.length;
    ObjectId[] od = new ObjectId[len];
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Get the set of same individuals which this individual belongs to.
   */
  public Collection getSameSet(){
    if(sameSet==null){
      sameSet = new Vector();
      sameSet.add(this.getID());
    }
    return sameSet;
  }

  /**
   * Set the set of same individuals which this individual belongs to.
   */
  public void setSameSet(Collection c){
    sameSet = c;
  }

  /**
   * Add a id which is differen from this individual.
   */
  public void addDifferentFrom(ObjectId id){
    differentFrom.add(id);
  }

  /**
   * Delete a id which is differen from this individual.
   */
  public void deleteDifferentFrom(ObjectId id){
    differentFrom.remove(id);
  }

  /**
   * Get all the ids which are differen from this individual.
   */
  public ObjectId[] getAllDifferentFrom(){
    if(differentFrom.size()<=0)
      return null;
    Object[] oa = differentFrom.toArray();
    int len = oa.length;
    ObjectId[] od = new ObjectId[len];
    System.arraycopy(oa,0,od,0,len);
    return od;
  }


  private Vector source;
  private Hashtable properties;
  private Collection sameSet;
  private Vector sameAs, differentFrom;

}