/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.simplememory;
import edu.lehigh.swat.hawk.core.ObjectId;
import java.util.Vector;

/**
 * A simple implementation of core.OntClass in memory.
 * @author Z. Pan
 * @version 1.0
 */

public class OntClass extends OntObject
    implements edu.lehigh.swat.hawk.core.OntClass{


  /**
   * Constructor
   */
  public OntClass() {
    subClasses = new Vector();
    superClasses = new Vector();
    complementOf = new Vector();
    unionOf = new Vector();
    oneOf = new Vector();
    intersectionOf = new Vector();
    equivalentClasses = new Vector();
    propertyRestrictions = new Vector();
    disjointWith = new Vector();
    deprecated = false;
    satisfiable = null;
  }

  /**
   * Add a direct subClass.
   * $memo: currently we only set bi-directional links to those classes who are
   * in the same document.
   */
  public void addSubClass(ObjectId c){
    if(!subClasses.contains(c)){
      subClasses.add(c);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntClass cc = ont.getOntClass(c);
      if(cc!=null)
        cc.addSuperClass(this.getID());
    }
  }

  /**
   * get the direct subClasses
   */
  public ObjectId[] getDirectSubClasses(){
    int len = subClasses.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = subClasses.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Delete a direct subClass.
   */
  public void deleteSubClass(ObjectId c){
    if(subClasses.contains(c)){
      subClasses.remove(c);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntClass cc = ont.getOntClass(c);
      if (cc != null)
        cc.deleteSuperClass(this.getID());
    }
  }

  /**
   * Add a direct super Class.
   */
  public void addSuperClass(ObjectId c){
    if(!superClasses.contains(c)){
      superClasses.add(c);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntClass cc = ont.getOntClass(c);
      if(cc!=null)
        cc.addSubClass(this.getID());
    }

  }

  /**
   * get the direct super Classes
   */
  public ObjectId[] getDirectSuperClasses(){
    int len = superClasses.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = superClasses.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * delete a direct super Class
   */
  public void deleteSuperClass(ObjectId c){
    if(superClasses.contains(c)){
      superClasses.remove(c);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntClass cc = ont.getOntClass(c);
      if (cc != null)
        cc.deleteSubClass(this.getID());
    }
  }

  /**
   * Add a property restriction.
   */
  public void addPropertyRestriction(ObjectId r){
    propertyRestrictions.add(r);
  }

  /**
   * Delete a property restriction.
   */
  public void deletePropertyRestriction(ObjectId r){
    propertyRestrictions.remove(r);
  }

  /**
   * Get the property restrictions.
   */
  public ObjectId[] getPropertyRestrictions(){
    int len = propertyRestrictions.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = propertyRestrictions.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add a equivalent class.
   */
  public void addEquivalentClass(ObjectId ec){
    if(!equivalentClasses.contains(ec)&&!ec.equals(this.getID()))
      equivalentClasses.add(ec);
  }

  /**
   * Delete a equivalent class.
   */
  public void deleteEquivalentClass(ObjectId ec){
    equivalentClasses.remove(ec);
  }

  /**
   * Get the equivalent classes.
   */
  public ObjectId[] getEquivalentClasses(){
    int len = equivalentClasses.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = equivalentClasses.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add an 'UnionOf' class.
   */
  public void addUnionOf(ObjectId u){
    unionOf.add(u);
  }

  /**
   * Delete an 'UnionOf' class.
   */
  public void deleteUnionOf(ObjectId u){
    unionOf.remove(u);
  }

  /**
   * Get the 'IntersectionOf' classes.
   */
  public ObjectId[] getUnionOfs(){
    int len = unionOf.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = unionOf.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add an 'IntersectionOf' class or restriction.
   */
  public void addIntersectionOf(ObjectId i){
    intersectionOf.add(i);
  }

  /**
   * Delete an 'IntersectionOf' class or restriction.
   */
  public void deleteIntersectionOf(ObjectId i){
    intersectionOf.remove(i);
  }

  /**
   * Get the 'IntersectionOf' classes or restrictions.
   */
  public ObjectId[] getIntersectionOfs(){
    int len = intersectionOf.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = intersectionOf.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add an 'OneOf' object.
   */
  public void addOneOf(ObjectId one){
    oneOf.add(one);
  }

  /**
   * Delete an 'OneOf' object.
   */
  public void deleteOneOf(ObjectId one){
    oneOf.remove(one);
  }

  /**
   * Get the 'OneOf' objects.
   */
  public ObjectId[] getOneOfs(){
    int len = oneOf.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = oneOf.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add an 'ComplementOf' class or restriction.
   */
  public void addComplementOf(ObjectId i){
    complementOf.add(i);
  }

  /**
   * Delete an 'ComplementOf' class or restriction.
   */
  public void deleteComplementOf(ObjectId i){
    complementOf.remove(i);
  }

  /**
   * Get the 'ComplementOf' classes or restrictions.
   */
  public ObjectId[] getComplementOfs(){
    int len = complementOf.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = complementOf.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add a 'disjointWith' class.
   */
  public void addDisjointWith(ObjectId id){
    disjointWith.add(id);
  }

  /**
   * Delete a 'disjointWith' class.
   */
  public void deleteDisjointWith(ObjectId id){
    disjointWith.remove(id);
  }

  /**
   * Get the disjointWith classes.
   */
  public ObjectId[] getDisjointWith(){
    int len = disjointWith.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = disjointWith.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Test if this class is deprecated.
   */
  public boolean isDeprecated(){
    return deprecated;
  }

  /**
   * Set the class to be deprecated.
   */
  public void setDeprecated(boolean dep){
    deprecated = dep;
  }

  /**
   * Set the satisfiability of this class.
   */
  public void setSatisfiable(Boolean sat){
    satisfiable = sat;
  }

  /**
   * Test if this class is satisfiable.
   */
  public Boolean isSatisfiabele(){
    return satisfiable;
  }

  private Boolean satisfiable;
  private boolean deprecated;
  private Vector subClasses, superClasses, complementOf, unionOf,
                 oneOf, intersectionOf, equivalentClasses, disjointWith,
                 propertyRestrictions;

}