/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/

package edu.lehigh.swat.hawk.storage.simplememory;
import edu.lehigh.swat.hawk.core.ObjectId;
import java.util.Vector;
/**
 * A simple implementation of core.OntProperty in memory.
 * @author Z. Pan
 * @version 1.0
 */

public class OntProperty extends OntObject
    implements edu.lehigh.swat.hawk.core.OntProperty{

  /**
   * Constructor
   */
  public OntProperty() {
    transitive = false;
    symetric = false;
    functional = false;
    inverseFunctional = false;
    deprecated = false;
    datatype = null;
    annotation = false;
    inverseOf = new Vector();
    subProperties = new Vector();
    superProperties = new Vector();
    equivalent = new Vector();
  }

  /**
   * get the domain
   */
  public ObjectId getDomain(){
    return domain;
  }

  /**
   * set the domain
   */
  public void setDomain(ObjectId c){
    domain = c;
  }

  /**
   * get the range
   */
  public ObjectId getRange(){
    return range;
  }

  /**
   * set the domain
   */
  public void setRange(ObjectId c){
    range = c;
  }

  /**
   * test if this property is transitive
   */
  public boolean isTransitive(){
    return transitive;
  }

  /**
   * set the property to be transitive or not
   */
  public void setTransitive(boolean b){
    transitive = b;
  }

  /**
   * test if this property is symetric
   */
  public boolean isSymmetric(){
    return symetric;
  }

  /**
   * set the property to be symetric or not
   */
  public void setSymmetric(boolean b){
    symetric = b;
  }

  /**
   * add a direct subProperty
   * $memo: currently we only set bi-directional links to those porperties who are
   * in the same document.
   */
  public void addSubProperty(ObjectId p){
    if(!subProperties.contains(p)){
      subProperties.add(p);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntProperty pp = ont.getProperty(p);
      if(pp!=null)
        pp.addSuperProperty(this.getID());
    }

  }

  /**
   * get the direct subProperties
   */
  public ObjectId[] getDirectSubProperties(){
    int len = subProperties.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = subProperties.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * delete a direct subProperty
   */
  public void deleteSubProperty(ObjectId p){
    if(subProperties.contains(p)){
      subProperties.remove(p);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntProperty pp = ont.getProperty(p);
      if(pp!=null)
        pp.deleteSuperProperty(this.getID());
    }

  }

  /**
   * add a direct super Property
   */
  public void addSuperProperty(ObjectId p){
    if(!superProperties.contains(p)){
      superProperties.add(p);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntProperty pp = ont.getProperty(p);
      if(pp!=null)
        pp.addSubProperty(this.getID());
    }

  }

  /**
   * get the direct super Properties
   */
  public ObjectId[] getDirectSuperProperties(){
    int len = superProperties.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = superProperties.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * delete a direct super Property
   */
  public void deleteSuperProperty(ObjectId p){
    if(superProperties.contains(p)){
      superProperties.remove(p);
      edu.lehigh.swat.hawk.core.Ontology ont = this.getOntology();
      edu.lehigh.swat.hawk.core.OntProperty pp = ont.getProperty(p);
      if(pp!=null)
        pp.deleteSubProperty(this.getID());
    }
  }

  /**
   * Test if this property is deprecated.
   */
  public boolean isDeprecated(){
    return deprecated;
  }

  /**
   * Set the property to be deprecated.
   */
  public void setDeprecated(boolean dep){
    deprecated = dep;
  }

  /**
   * Test if this property is functional.
   */
  public boolean isFunctional(){
    return functional;
  }

  /**
   * Set the property to be functional.
   */
  public void setFunctional(boolean f){
    functional = f;
  }

  /**
   * Test if this property is inverse functional.
   */
  public boolean isInverseFunctional(){
    return inverseFunctional;
  }

  /**
   * Set the property to be inverse functional.
   */
  public void setInverseFuncional(boolean f){
    inverseFunctional = f;
  }

  /**
   * Add a 'inverseOf' property
   */
  public void addInverseOf(ObjectId p){
    if(!inverseOf.contains(p))
      inverseOf.add(p);
  }

  /**
   * delete a 'inverseOf' property
   */
  public void deleteInverseOf(ObjectId p){
    inverseOf.remove(p);
  }

  /**
   * get the 'inverseOf' properties
   */
  public ObjectId[] getInverseOf(){
    int len = inverseOf.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = inverseOf.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }

  /**
   * Add a equivalent property.
   */
  public void addEquivalentProperty(ObjectId ec){
    if(!equivalent.contains(ec)&&!ec.equals(this.getID()))
      equivalent.add(ec);
  }

  /**
   * Delete a equivalent property.
   */
  public void deleteEquivalentProperty(ObjectId r){
    equivalent.remove(r);
  }

  /**
   * Get the equivalent properties.
   */
  public ObjectId[] getEquivalentProperties(){
    int len = equivalent.size();
    ObjectId[] od = new ObjectId[len];
    Object[] oa = equivalent.toArray();
    System.arraycopy(oa,0,od,0,len);
    return od;
  }


  /**
   * Tell this property is ObjectProperty or DatatypeProperty.
   * Null means unknown.
   */
  public Boolean isDatatype(){
    return datatype;
  }

  /**
   * Set this property to be DatatypeProperty or ObjectProperty.
   */
  public void setDatatype(boolean b){
    datatype = new Boolean(b);
  }
  /**
   * Tell this property is AnnotationProperty or not.
   *
   */
  public boolean isAnnotation(){
    return annotation;
  }

  /**
   * Set this property to be DatatypeProperty or not.
   */
  public void setAnnotation(boolean b){
    annotation = b;
  }

  private boolean transitive, symetric, functional, inverseFunctional,
                   deprecated, annotation;
  private Boolean datatype;

  private Vector inverseOf, subProperties, superProperties,equivalent;
  private ObjectId domain, range;



}
