/*
*Copyright: Copyright (c) 2003-2004, Lehigh University. All rights reserved.
*License Statement: The GNU General Public License
*/


package edu.lehigh.swat.hawk.storage.simplememory;
import java.util.*;
import edu.lehigh.swat.hawk.core.ObjectId;
import edu.lehigh.swat.hawk.core.DocId;
/**
 * A simple implementation of core.ontology in memory.
 * @author Z. Pan
 * @version 1.0
 */

public class Ontology extends Document implements edu.lehigh.swat.hawk.core.Ontology{

  /**
   * Constructor
   */
  public Ontology() {
    classes = new Hashtable();
    properties = new Hashtable();
    restrictions = new Hashtable();
    compatiblePriorOnts = new Vector();
    priorOnts = new Vector();
    version = null;
    importedClasses = new Hashtable();
    importedProperties = new Hashtable();
  }

  /**
   * add a class to this ontology
   */
  public void addOntClass(edu.lehigh.swat.hawk.core.OntClass c){
    classes.put(c.getID(),c);
  }

  /**
   * get the specified class
   */
  public edu.lehigh.swat.hawk.core.OntClass getOntClass(ObjectId id){
    if(classes.containsKey(id))
      return (edu.lehigh.swat.hawk.core.OntClass)classes.get(id);
    if(importedClasses.containsKey(id)){
      return (edu.lehigh.swat.hawk.core.OntClass) importedClasses.get(id);
    }
    return null;
  }

  /**
   * delete the specified class
   */
  public void deleteOntClass(ObjectId id){
    if(classes.containsKey(id))
      classes.remove(id);
  }

  /**
   * get all the classes in this ontology
   */
  public edu.lehigh.swat.hawk.core.OntClass[] getAllClasses(){
    Collection c = classes.values();
    Object[] oa = c.toArray();
    int len = oa.length;
    edu.lehigh.swat.hawk.core.OntClass[] oc = new edu.lehigh.swat.hawk.core.OntClass[len];
    System.arraycopy(oa,0,oc,0,len);
    return oc;

  }

  /**
   * Get the Iterator of all the classes.
   */
  public Iterator getClassIterator(){
    return classes.values().iterator();
  }


  /**
   * add a property to this ontology
   */
  public void addProperty(edu.lehigh.swat.hawk.core.OntProperty p){
    properties.put(p.getID(),p);
  }

  /**
   * get the specified property
   */
  public edu.lehigh.swat.hawk.core.OntProperty getProperty(ObjectId id){
    if(properties.containsKey(id))
      return (edu.lehigh.swat.hawk.core.OntProperty)properties.get(id);
    else if(importedProperties.containsKey(id))
      return (edu.lehigh.swat.hawk.core.OntProperty)importedProperties.get(id);
    return null;

  }

  /**
   * delete a specified property
   */
  public void deleteProperty(ObjectId id){
    properties.remove(id);
  }

  /**
   * get all the properties in this ontology
   */
  public edu.lehigh.swat.hawk.core.OntProperty[] getAllProperties(){
    Collection c = properties.values();
    Object[] oa = c.toArray();
    int len = oa.length;;
    edu.lehigh.swat.hawk.core.OntProperty[] oc = new edu.lehigh.swat.hawk.core.OntProperty[len];
    System.arraycopy(oa,0,oc,0,len);
    return oc;
  }

  /**
   * Get the Iterator of all the properties.
   */
  public Iterator getPropertyIterator(){
    return properties.values().iterator();
  }


  /**
   * add a property to this ontology
   */
  public void addRestriction(edu.lehigh.swat.hawk.core.PropertyRestriction p){
    restrictions.put(p.getID(),p);
  }

  /**
   * get the specified property
   */
  public edu.lehigh.swat.hawk.core.PropertyRestriction getRestriction(ObjectId id){
    if(restrictions.containsKey(id))
      return (edu.lehigh.swat.hawk.core.PropertyRestriction)restrictions.get(id);
    else
      return null;
  }

  /**
   * delete a specified property
   */
  public void deleteRestriction(ObjectId id){
    restrictions.remove(id);
  }

  /**
   * get all the properties in this ontology
   */
  public edu.lehigh.swat.hawk.core.PropertyRestriction[] getAllRestrictions(){
    Collection c = restrictions.values();
    Object[] oa = c.toArray();
    int len = oa.length;
    edu.lehigh.swat.hawk.core.PropertyRestriction[] oc = new edu.lehigh.swat.hawk.core.PropertyRestriction[len];
    System.arraycopy(oa,0,oc,0,len);
    return oc;
  }

  /**
   * Get the priorVersion ontologies which are backwardCompatibleWith
   */
  public DocId[] getCompatiblePriorOnts(){
    if(compatiblePriorOnts.size()<=0)
      return null;
    else{
     Object[] oa = compatiblePriorOnts.toArray();
     int len = oa.length;
     DocId[] oc = new DocId[len];
     System.arraycopy(oa,0,oc,0,len);
     return oc;
    }
  }

  /**
   * Add a priorVersion ontology which is backwardCompatibleWith.
   */
  public void addCompatiblePriorOnts(DocId id){
    compatiblePriorOnts.add(id);
  }

  /**
   * Delete a priorVersion ontology which is backwardCompatibleWith.
   */
  public void deleteCompatiblePriorOnts(DocId id){
    compatiblePriorOnts.remove(id);
  }

  /**
   * Get the priorVersion ontologies may be incompatibleWith
   */
  public DocId[] getPriorOnts(){
    if(priorOnts.size()<=0)
      return null;
    else{
     Object[] oa = priorOnts.toArray();
     int len = oa.length;
     DocId[] oc = new DocId[len];
     System.arraycopy(oa,0,oc,0,len);
     return oc;
    }
  }

  /**
   * Add a priorVersion ontology may be incompatibleWith.
   */
  public void addPriorOnts(DocId id){
    priorOnts.add(id);
  }

  /**
   * Delete a priorVersion ontology may be incompatibleWith.
   */
  public void deletePriorOnts(DocId id){
    priorOnts.remove(id);
  }

  /**
   * get the version of this ontology
   */
  public String getVersion(){
    return version;
  }

  /**
   * Set the version of this ontology.
   */
  public void setVersion(String ver){
    version = ver;
  }

  /**
   * Create a Class object and set its ontology.
   */
  public edu.lehigh.swat.hawk.core.OntClass createClass(){
    OntClass cls = new OntClass();
    cls.setOntology(this);
    return cls;
  }

  /**
   * Create a Property object and set its ontology.
   */
  public edu.lehigh.swat.hawk.core.OntProperty createProperty(){
    OntProperty property = new OntProperty();
    property.setOntology(this);
    return property;

  }

  /**
   * Create a PropertyRestriction object and set its ontology.
   */
  public edu.lehigh.swat.hawk.core.PropertyRestriction createRestriction(){
    PropertyRestriction pr = new PropertyRestriction();
    pr.setOntology(this);
    return pr;
  }

  /**
   * To see if ontology has the class.
   */
  public boolean hasClass(String name){
    Enumeration e = classes.keys();
    while(e.hasMoreElements()){
      ObjectId oid = (ObjectId)e.nextElement();
      if(oid.getLocalName().equalsIgnoreCase(name))
        return true;
    }
    return false;
  }

  /**
   * To see if ontology has the property.
   */
  public boolean hasProperty(String name){
    Enumeration e = properties.keys();
    while(e.hasMoreElements()){
      ObjectId oid = (ObjectId)e.nextElement();
      if(oid.getLocalName().equalsIgnoreCase(name))
        return true;
    }
    return false;
  }

  /**
   * Add an imported class.
   */
  public void addImportedClass(ObjectId cid, edu.lehigh.swat.hawk.core.OntClass c){
    importedClasses.put(cid,c);
  }

  /**
   * Add an imported property.
   */
  public void addImportedProperty(ObjectId pid, edu.lehigh.swat.hawk.core.OntProperty p){
    importedProperties.put(pid,p);
  }


  /**
   * Get All the classes plus the imported Classes.
   */
  public edu.lehigh.swat.hawk.core.OntClass[] getAllPlusImportedClasses(){
    edu.lehigh.swat.hawk.core.OntClass[] ac = this.getAllClasses();
    int num = ac.length;
    Collection c = importedClasses.values();
    Object[] oa = c.toArray();
    int len = oa.length;
    edu.lehigh.swat.hawk.core.OntClass[] oc = new edu.lehigh.swat.hawk.core.OntClass[len+num];
    System.arraycopy(ac,0,oc,0,num);
    System.arraycopy(oa,0,oc,num,len);
    return oc;
  }

  /**
   * Get All the classes plus the imported properties.
   */
  public edu.lehigh.swat.hawk.core.OntProperty[] getAllPlusImportedProperties(){
    edu.lehigh.swat.hawk.core.OntProperty[] ac = this.getAllProperties();
    int num = ac.length;
    Collection c = importedProperties.values();
    Object[] oa = c.toArray();
    int len = oa.length;
    edu.lehigh.swat.hawk.core.OntProperty[] oc = new edu.lehigh.swat.hawk.core.OntProperty[len+num];
    System.arraycopy(ac,0,oc,0,num);
    System.arraycopy(oa,0,oc,num,len);
    return oc;
  }

  /**
   * To tell if the ontology imorted specified class.
   */
  public boolean importedClass(ObjectId cid){
    Enumeration e = importedClasses.keys();
    while(e.hasMoreElements()){
      ObjectId oid = (ObjectId)e.nextElement();
      if(cid.equals(oid))
        return true;
    }
    return false;
  }

  /**
   * To tell if the ontology imported specified property.
   */
  public boolean importedProperty(ObjectId pid){
    Enumeration e = importedProperties.keys();
    while(e.hasMoreElements()){
      ObjectId oid = (ObjectId)e.nextElement();
      if(pid.equals(oid))
        return true;
    }
    return false;
  }


  /**
   * Get the imported class.
   */
//  public edu.lehigh.swat.hawk.core.OntClass getImportedClass(ObjectId id){
//    if(importedClasses.containsKey(id))
//      return (edu.lehigh.swat.hawk.core.OntClass)importedClasses.get(id);
//    else
//      return null;
//  }

  /**
   * Get the imported property.
   */
//  public edu.lehigh.swat.hawk.core.OntProperty getImportedProperty(ObjectId id){
//    if(importedProperties.containsKey(id))
//      return (edu.lehigh.swat.hawk.core.OntProperty)importedProperties.get(id);
//    else
//      return null;
//  }


  private Hashtable classes, properties, restrictions, importedClasses, importedProperties;
  private Vector priorOnts, compatiblePriorOnts;
  private String version;
}
